/***************************************************************************
 *   Copyright (C) 2005 by Jean-Baptiste Ferdy                             *
 *   ferdy@univ-montp2.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "commande_matrice.h"
#include "form_commande_matrice.h"

#include <qpushbutton.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qcheckbox.h>

commande_matrice::commande_matrice(int nl,int nc,QWidget *parent, const char *name)
 : Form_commande_matrice(parent, name)
{
  QObject::connect(comboBox_commande,SIGNAL(activated(int)),this,SLOT(autorise_position()));
  QObject::connect(pushButton_ok,SIGNAL(clicked()),this,SLOT(ok()));
  QObject::connect(pushButton_annuler,SIGNAL(clicked()),this,SLOT(annuler()));

  spinBox_l_start->setMaxValue(nl);
  spinBox_l_start->setMinValue(1);
  spinBox_l_stop->setMaxValue(nl);
  spinBox_l_stop->setMinValue(1);
  
  checkBox_rebours_l_stop->setChecked(true);
  spinBox_l_stop->setValue(1);
  
  spinBox_c_start->setMaxValue(nc);
  spinBox_c_start->setMinValue(1);
  spinBox_c_stop->setMaxValue(nc);
  spinBox_c_stop->setMinValue(1);
  
  checkBox_rebours_c_stop->setChecked(true);
  spinBox_c_stop->setValue(1);
  
  spinBox_pos->setMaxValue(nc);
  spinBox_pos->setMinValue(-nc);
  spinBox_pos->setValue(0);
  
  QStringList list;
  list << "ligne";
  list << "colone";
  list << "diagonale";
  list << "bloc";  
  list << "additionner";
  list << "soustraire";
  list << "multiplier";
  list << "diviser";
  comboBox_commande->insertStringList(list);
  autorise_position();
}


commande_matrice::~commande_matrice()
{  
}

void commande_matrice::autorise_position(bool b) {  
  spinBox_l_start->setEnabled(!b);
  spinBox_c_start->setEnabled(!b);
  spinBox_l_stop->setEnabled(!b);
  spinBox_c_stop->setEnabled(!b);
  checkBox_rebours_l_start->setEnabled(!b);
  checkBox_rebours_c_start->setEnabled(!b);
  checkBox_rebours_l_stop->setEnabled(!b);
  checkBox_rebours_c_stop->setEnabled(!b);
  spinBox_pos->setEnabled(b);
  }

void commande_matrice::autorise_ligne(bool b) {  
  spinBox_l_start->setEnabled(true);
  spinBox_l_stop->setEnabled(b);  
  checkBox_rebours_l_start->setEnabled(true); 
  checkBox_rebours_l_stop->setEnabled(b);    
  }
  
void commande_matrice::autorise_colone(bool b) {  
  spinBox_c_start->setEnabled(true);
  spinBox_c_stop->setEnabled(b);  
  checkBox_rebours_c_start->setEnabled(true); 
  checkBox_rebours_c_stop->setEnabled(b);    
  }
    
void commande_matrice::autorise_position() {
  QString buf = comboBox_commande->currentText();
  if(buf=="diagonale") autorise_position(true);
  else autorise_position(false);
  if(buf=="ligne")  autorise_ligne(false);
  if(buf=="colone") autorise_colone(false);
  }
    
QString commande_matrice::get_commande() {
  QString bal,tag,pos,buf;
  bal = comboBox_commande->currentText();
  if(bal=="additionner")       {bal="operation"; tag=" type=+";}
  if(bal=="soustraire")   {bal="operation"; tag=" type=-";}
  if(bal=="multiplier") {bal="operation"; tag=" type=*";}
  if(bal=="diviser")       {bal="operation"; tag=" type=/";}
    
  if(spinBox_pos->isEnabled()) {
    pos += " pos=" + spinBox_pos->text();
    }
  else {
    int test;
    if(checkBox_rebours_l_stop->isChecked())  test = spinBox_l_stop->maxValue()+1-spinBox_l_stop->value();
    else                                      test = spinBox_l_stop->value();
    if(checkBox_rebours_l_start->isChecked()) test -= spinBox_l_start->maxValue()+1-spinBox_l_start->value();
    else                                      test -= spinBox_l_start->value();
    
    if(spinBox_l_start->isEnabled() && !(spinBox_l_stop->isEnabled() && test==spinBox_l_stop->maxValue()-spinBox_l_start->minValue())) {
      pos += " l=";
      if(checkBox_rebours_l_start->isChecked()) pos+="-";
      pos += spinBox_l_start->text();
      if(spinBox_l_stop->isEnabled()) {
        pos += ":";
        if(checkBox_rebours_l_stop->isChecked()) pos+="-";
        pos += spinBox_l_stop->text();
	}
      }
      
    if(checkBox_rebours_c_stop->isChecked())  test = spinBox_c_stop->maxValue()+1-spinBox_c_stop->value();
    else                                      test = spinBox_c_stop->value();
    if(checkBox_rebours_c_start->isChecked()) test -= spinBox_c_start->maxValue()+1-spinBox_c_start->value();
    else                                      test -= spinBox_c_start->value(); 
    
    if(spinBox_c_start->isEnabled() && !(spinBox_c_stop->isEnabled() && test ==spinBox_c_stop->maxValue()-spinBox_c_start->minValue())) {      
      pos += " c=";
      if(checkBox_rebours_c_start->isChecked()) pos+="-";
      pos += spinBox_c_start->text();
      if(spinBox_c_stop->isEnabled()) {
        pos+=":";
        if(checkBox_rebours_c_stop->isChecked()) pos+="-";
        pos += spinBox_c_stop->text();
        }
      }  
    }
  
  buf = "<"+bal+tag+pos+">";
  buf += lineEdit_contenu->text();
  buf += "</"+bal+">";
  
  return(buf);
  }

void commande_matrice::ok() {
  accept();
  }

void commande_matrice::annuler() {
  reject();
  }

//#include "commande_matrice.moc"
