/***************************************************************************
 *   Copyright (C) 2004 by Ferdy Jean-Baptiste                             *
 *   ferdy@univ-montp2.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef COORDONNEES_H
#define COORDONNEES_H

#include "model.h"
#include <qpainter.h>
#include <qpixmap.h>

/**
@author Ferdy Jean-Baptiste
*/
class coordonnees{
public:
    coordonnees(QPixmap* pict);
    ~coordonnees();
    
    double    get_xmin();
    double    get_xmax();
    double    get_ymin();
    double    get_ymax();
    int       get_xstep();
    int       get_ystep();
    int       get_automatique_x();
    int       get_automatique_y();
    
    void      set_automatique_x(int i);
    void      set_automatique_y(int i);
    
    void      set_xmin(double x);
    void      set_xmax(double x);
    void      set_xstep(int i);
    void      set_ymin(double y);
    void      set_ymax(double y);
    void      set_ystep(int i);
    
    
    void   calcul_tmax_xymax(modele* m);
    void   calcul_xymax(modele* m);
    void   calcul_xmax(modele* m);
    void   calcul_echelle();
    void   calcul_echelle(QPixmap* pict);
    
    void   set_marge(double g_x,double d_x,double g_y,double d_y);
    
    int    sc_x(double x);			//affichage standard, en plan de phase, ou de priode
    int    sc_y(double y);
    double usc_x(int x);
    double usc_y(int y);
    
    int    sc_x(double x,double y,double z);	//affichage diagramme de de Finetti
    int    sc_y(double x,double y,double z);
    double usc_x(int x,int y);
    double usc_y(int x,int y);
    double usc_z(int x,int y);
        
    void   troncation_xy(double* x1,double* y1,double* x2,double* y2);
    void   trace_ligne(QPainter* p,double x1,double y1,double x2,double y2);
    void   trace_ligne(QPainter* p,double *pt1,double *pt2);
    void   trace_ligne(QPainter* p,double x1,double y1,double z1,double x2,double y2,double z2);
    
    void   trace_fleche(QPainter* p,double x1,double y1,double x2,double y2);
    
    void   trace_courbe(QPainter* p,int x,int y,int nb_pt,double** v);
    
    void   trace_courbe(QPainter* p,int nb_var,int nb_pt,double** v);
    void   trace_courbe(QPainter* p,int nb_var,int nb_pt,double** v,int style);
    
    void   trace_courbe_fleche(QPainter* p,int x,int y,int nb_pt,double** v);
    
    void   trace_point(QPainter* p,double x,double y);
    void   trace_point(QPainter* p,double* x);
    void   trace_point(QPainter* p,double x,double y,double z);
    void   trace_dble_point(QPainter* p,double x,double y);
    void   trace_dble_point(QPainter* p,double* x);
    void   trace_dble_point(QPainter* p,double x,double y,double z);

    void   trace_curseur_equilibre(QPainter* p,double* x,int var_nb,int stable);
    void   trace_point_equilibre(QPainter* p,double* pt,int nb,int stable);

    int    point_valide(double x,double y);
    
    
    void   dessine_axe(QPainter *p,char* x_nom,char* y_nom);
    void   dessine_legende(QPainter *p,modele* m);
    
    void   dessine_triangle(QPainter *p,char* x_nom,char* y_nom,char* z_nom);
    
private:
	int    automatique_x;
	int    automatique_y;
	
	double xmin;
	double xmax;
	double ymin;
	double ymax;	
	int    xstep;
	int    ystep;
	double dx;
	double dy;
	double scale_x;
	double scale_y;
	
	int    w;
	int    h;
	int    marge_g_x;
	int    marge_d_x;
	int    marge_g_y;
	int    marge_d_y;
	int    lx;
	int    ly;
};

void couleur(QPainter *p,int sp,int sp_nb);
void couleur(QPainter *p,int sp,int sp_nb,int style);
#endif
