/***************************************************************************
 *   Copyright (C) 2005 by Jean-Baptiste Ferdy                             *
 *   ferdy@univ-montp2.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "distribution.h" 
 
extern FILE*  FICHIER_SORTIE;

//---------------------------------------------------------------
//construteur et destructeur
//---------------------------------------------------------------
distribution::distribution(int xsymb,fonction* xf_m,fonction* xf_p) {  
  symb = xsymb;
  type = type_distribution(symb);
  f_m  = xf_m;
  f_p  = xf_p;
  
  int test = 0;
  if(test==0 && f_m!=NULL && nb_arg_correct_distribution(type,f_m->get_nb())==1) {
    test = 1;
    f_p  = NULL;
    n    = f_m->get_nb();
    }
  if(test==0 && f_p!=NULL && nb_arg_correct_distribution(type,f_p->get_nb())==1) {    
    test = 1;
    f_m  = NULL;
    n    = f_p->get_nb();
    }
  if(test == 1) { //la distribution est valide
    f_eval   = get_fonction_evaluation_distribution(type);
    f_constr = get_fonction_construction_distribution(type);
    p = (double*) malloc(n*sizeof(double));
    }
  else { //la distribution n'est pas valide
    type = -1; n = -1;
    f_m  = NULL; f_p  = NULL;
    f_eval = NULL; f_constr = NULL;
    p = NULL;
    }
  }

distribution::distribution(char* texte,int var_nb,char** var_nom,double* var,int par_nb,char** par_nom,double* par) {
	symb=-1;
	type=-1;
	n   = 0;
	f_m=NULL;
	f_p=NULL;
	p = NULL;
	
	char* buf;
	//lecture du type de distribution
	buf = lire_balise(texte,(char*)"famille");
	if(buf != NULL) {
	  symb = symbole_distribution(buf);  
	  
	  if(symb=='?') {
	    symb=-1; type=-1; n = -1;
	    fprintf(FICHIER_SORTIE,"ERREUR distribution::distribution - distribution %s inconnue.\n",buf); 
	    free(buf); 
	    return;
	    }
	  type = type_distribution(buf);
	  f_constr = get_fonction_construction_distribution(symb);
	  f_eval   = get_fonction_evaluation_distribution(symb);  
	  if(f_eval == NULL) {
	    symb=-1; type=-1; n = -1;
	    fprintf(FICHIER_SORTIE,"ERREUR distribution::distribution - impossible de trouver une fonction d'valuation pour la distribution %s.\n",buf); 
	    return;
	    }
	  free(buf);
	  }
	
	//lecture des moments
	buf = lire_balise(texte,(char*)"moment");	
	if(buf!=NULL) {
		if(f_constr==NULL) { //la distribution ne peut pas tre dfinie par ses moments
		  fprintf(FICHIER_SORTIE,"ERREUR distribution::distribution - la distribution %s ne peut tre dfinie par ses moments.\n",nom_distribution(symb));
		  symb = -1; type = -1; n = -1;
		  return;
		  }
		char** bufs1 = NULL;
                n = str2tab(buf,&bufs1);
		if(n>0) {
		  f_m = new fonction(n,var_nb,par_nb,var_nom,par_nom,var,par,bufs1);
		  if(f_m->get_nb()==0) {
			fprintf(FICHIER_SORTIE,"ERREUR distribution::distribution - impossible de lire les moments de la distribution.\n");
			symb = -1; type = -1; n = -1;
			}
		  }
		free(buf); 
		}
		
	//lecture des paramtres
	buf = lire_balise(texte,(char*)"parametre");
	if(buf!=NULL) { 		
	        char** bufs1 = NULL;
                n = str2tab(buf,&bufs1);
		if(n>0) {
		  f_p = new fonction(n,var_nb,par_nb,var_nom,par_nom,var,par,bufs1);
		  if(f_p->get_nb()==0) {
			fprintf(FICHIER_SORTIE,"ERREUR distribution::distribution - impossible de lire les paramtres de la distribution.\n");
			symb = -1; type = -1; n = -1;
			}
		  }
		free(buf);
		}
		
   if(this->valide()==-1) {
        if(nb_arg_correct_distribution((char)symb,n)==0) fprintf(FICHIER_SORTIE,"ERREUR distribution::distribution - la distribution %s ne peut tre dfinie par %d paramtre(s).\n",nom_distribution((char)symb),n);
	
        symb = -1; type = -1; n = -1;
	if(f_m!=NULL) delete f_m; f_m=NULL;
	if(f_p!=NULL) delete f_p; f_p=NULL;
	p = NULL;
     }
   else {
     p = (double*) malloc(n*sizeof(double));
     for(int i=0;i<n;i++) p[i] = 0.0;   
     }
}

distribution::distribution() {
	symb =-1; type =-1; n = -1;
	f_m      = NULL;
	f_p      = NULL;
	f_eval   = NULL;
	f_constr = NULL;
	p        = NULL;
}

distribution::~distribution(){
	symb =-1; type =-1; n = -1;
	if(f_m!=NULL) delete f_m; f_m=NULL;
	if(f_p!=NULL) delete f_p; f_p=NULL;
	f_eval   = NULL;
	f_constr = NULL;
	if(p!=NULL) free(p);
}


//routines de base
//---------------------------------------------------------------
int distribution::valide() {
  if(symb==-1 || type==-1 || n<0) return(-1);  
  if(nb_arg_correct_distribution((char)symb,n)!=1) return(-1);  
  return(1);
  }
  
//evaluation
int distribution::eval(double* x) { 
 int test = 1;
 //calcul des paramtres
 if(f_m!=NULL)  {
   test = f_m->eval(NULL,p); 
   if(f_constr==NULL) {
     fprintf(FICHIER_SORTIE,"ERREUR distribution::distribution -- la fonction de construction  partir des moments n'est pas dfinie.\n");
     return(-1);
     }
   (void)(*f_constr)(n,p,&test); 
   }
 if(f_p!=NULL)  {
   test = f_p->eval(NULL,p);
   }
 
 //valuation
 if(test==1) {
   if(f_eval==NULL) {
     fprintf(FICHIER_SORTIE,"ERREUR distribution::distribution -- la fonction d'valuation n'est pas dfinie.\n");
     return(-1);
     }
   *x = (*f_eval)(n,p,&test);
   }
   
 return(test);
 }

void distribution::ecrire(char* var_nom) {
  fprintf(FICHIER_SORTIE,"%s suit une distribution %s\n",var_nom,nom_distribution((char)symb));  
  if(f_m!=NULL) {fprintf(FICHIER_SORTIE,"Moments de la distribution :\n");  f_m->ecrire();}
  if(f_p!=NULL) {fprintf(FICHIER_SORTIE,"Paramtres de la distribution :\n"); f_p->ecrire();}
  }
  
  

