/***************************************************************************
                          graphique.cpp  -  description
                             -------------------
    begin                : jeu avr 8 2004
    copyright            : (C) 2004 by Ferdy Jean-Baptiste
    email                : ferdy@univ-montp2.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <qmessagebox.h>
#include <qcursor.h>
#include <qtabwidget.h>

#include "ksim.h"
#include "coordonnees.h"
#include "graphique.h"

 Graphique::Graphique( QWidget *parent, const char *name )
    : QWidget( parent, name )
{   
    pere = parent;
    m    = NULL;
    type_affichage  = 0;
    calcul_en_cours = 0;
}

   
 void Graphique::paintEvent( QPaintEvent *) {
   refresh();
   };


extern coordonnees *COORD_COURANT;

double x_press,y_press,x_release,y_release;
int    left_press = 0;

//dplacement de la souris   
void Graphique::mouseMoveEvent( QMouseEvent *e) {
if(m==NULL || !m->valide()) return;

if(calcul_en_cours==1) {
	setCursor(WaitCursor); 
	return;
	}
if(type_affichage == 1) {  
    double x = e->x();
    double y = e->y();    
    if(left_press == 1) {               
      QPainter*  p = new QPainter();
      p->begin(this,true);
      p->setBrush(Qt::Dense7Pattern); 
      p->setPen(Qt::gray);     
      double w_press = COORD_COURANT->sc_x(x_press);
      double z_press = COORD_COURANT->sc_y(y_press);
      refresh();
      p->drawRect(w_press,z_press,x-w_press,y-z_press); 
      
      //est-ce que la touche Esc a t frappe ?
      grabKeyboard ();     
      }
    else {
      if(x<COORD_COURANT->get_xmin() || y<COORD_COURANT->get_ymin()) this->unsetCursor();
      else this->setCursor(CrossCursor);
      }
    return;
    };
    
if(type_affichage == 2) {
    double x = COORD_COURANT->usc_x(e->x(),e->y());
    double y = COORD_COURANT->usc_y(e->x(),e->y());
    double z = COORD_COURANT->usc_z(e->x(),e->y());    
    if(x<0.0||y<0.0||z<0.0||x>1.0||y>1.0||z>1.0) this->unsetCursor();
    else this->setCursor(CrossCursor);   
    return;
    };    
   this->unsetCursor();
   };



//zoom
void Graphique::mousePressEvent(QMouseEvent *e) {  
   if(m==NULL || !m->valide()) return;
   
   //bouton de droite : dfinition zone d'affichage
   if( e->button() == Qt::RightButton ) {
     if( type_affichage == 1) { //plan de phase
       left_press = 1;
       x_press = COORD_COURANT->usc_x(e->x());
       y_press = COORD_COURANT->usc_y(e->y());
       }
     }
   //bouton de gauche : lancement d'un nouveau calcul  
   if( e->button() == Qt::LeftButton ) {  
     if( type_affichage == 1) { //plan de phase
     double x = COORD_COURANT->usc_x(e->x());
     double y = COORD_COURANT->usc_y(e->y());
            
     if(x<COORD_COURANT->get_xmin() || y<COORD_COURANT->get_ymin()) return;
         
     QString buf = QString("Calculer une trajectoire  partir de :\n%1 = %2\n%3 = %4")
      .arg(m->get_var_nom(0))
      .arg(x)
      .arg(m->get_var_nom(1))
      .arg(y);
   
     switch( QMessageBox::warning( this, "KSimPop",buf,"Ok","Annuler", 0, 0, 1 ) )
      {
      case 0: {
        //rcupre les conditions initiales
        buf.setNum(x,'g',6);
        ((KSim*)pere)->get_table_variable()->setText(0,0,buf);
        buf.setNum(y,'g',6);
        ((KSim*)pere)->get_table_variable()->setText(1,0,buf);

        ((KSim*)pere)->lancer();
	
        break;
        }
      case 1: {// Annule
        break;
        }
      }
      return;
    };

    //diagramme de definetti
    if( type_affichage == 2 && m != NULL && m->valide()) {
    
      double x = COORD_COURANT->usc_x(e->x(),e->y());
      double y = COORD_COURANT->usc_y(e->x(),e->y());
      double z = COORD_COURANT->usc_z(e->x(),e->y());
    
      if(x<0.0||y<0.0||z<0.0||x>1.0||y>1.0||z>1.0) return;   
    
      QTable* tvar = ((KSim*)pere)->get_table_variable();    
      m->set_q_var(tvar);
      double s = m->get_var(0) + m->get_var(1) + m->get_var(2);
      x*=s; y*=s; z*=s;
        
      QString buf = QString("Calculer une trajectoire  partir de :\n%1 = %2\n%3 = %4\n%5 = %6\n")
      .arg(m->get_var_nom(0))
      .arg(x)
      .arg(m->get_var_nom(1))
      .arg(y)
      .arg(m->get_var_nom(2))
      .arg(z);

      switch( QMessageBox::warning( this, "KSimPop",buf,"Ok","Annuler", 0, 0, 1 ) )
      {
      case 0: {
        //rcupre les conditions initiales
        buf.setNum(x,'g',6);
        tvar->setText(0,0,buf);
        buf.setNum(y,'g',6);
        tvar->setText(1,0,buf);
        buf.setNum(z,'g',6);
        tvar->setText(2,0,buf);
	          
        //calcule la trajectoire
        ((KSim*)pere)->lancer();
		
        break;
        }
      case 1: {// Annule
        break;
        };
      };

      return;
    };
   }
   }
void Graphique::mouseReleaseEvent(QMouseEvent *e) {  
   if(m==NULL || !m->valide()) return;
   //libre le clavier
   releaseKeyboard ();
   
   if( type_affichage == 1 && left_press==1 && e->button() == Qt::RightButton ) { //plan de phase
     left_press = 0;
     x_release = COORD_COURANT->usc_x(e->x());
     y_release = COORD_COURANT->usc_y(e->y());
     double diag = sqrt(pow(COORD_COURANT->get_xmin()-COORD_COURANT->get_xmax(),2.0) + pow(COORD_COURANT->get_ymin()-COORD_COURANT->get_ymax(),2.0));
     if(sqrt(pow(x_press-x_release,2.0)+pow(y_press-y_release,2.0))<0.001*diag) return;
     if(x_press>x_release) {
      double z = x_press;
      x_press=x_release;
      x_release=z;
      }
     if(y_press>y_release) {
      double z = y_press;
      y_press=y_release;
      y_release=z;
      } 
   
     COORD_COURANT->set_xmin(x_press);
     COORD_COURANT->set_xmax(x_release);
     COORD_COURANT->set_ymin(y_press);
     COORD_COURANT->set_ymax(y_release);
     COORD_COURANT->set_automatique_x(0);
     COORD_COURANT->set_automatique_y(0);
     ((KSim*)pere)->rafraichir_affichage();     
     }      
   }

void Graphique::keyPressEvent ( QKeyEvent * e ) {
 if(e->key() == Qt::Key_Escape) {   
   left_press = 0;
   refresh();
   } 
 }
 
void Graphique::set(QWidget* parent) {
      //recupre l'adresse du widget appelant
      pere = parent;      
   }

void Graphique::refresh(qmodele *xm,int xtype_affichage) {
   m = xm;      
   
   type_affichage = xtype_affichage;
   if(calcul_en_cours==1) setCursor(WaitCursor); 
   else {	   
	   if(type_affichage==1 || type_affichage==2) this->setCursor(CrossCursor);
	   else this->unsetCursor();
   }
   
   if(m!=NULL) {
    bitBlt((QWidget*)this,0,0,m->pict,0,0,m->pict->width(),m->pict->height());
    }
   else {
   	QPainter*  p = new QPainter();
  	
	p->begin(this,true);
	QColor c; c.setRgb(255,255,255); p->setBrush(c);
  	p->drawRect(0,0,this->width(),this->height());
	p->end();
  	delete p;
	}
   
   }   

   void Graphique::refresh(int xtype_affichage) {
   type_affichage = xtype_affichage;
   if(calcul_en_cours==1) setCursor(WaitCursor); 
   else {	   
	   if(type_affichage==1 || type_affichage==2) this->setCursor(CrossCursor);
	   else this->unsetCursor();
   }      
   
   if(m!=NULL) {    
	bitBlt((QWidget*)this,0,0,m->pict,0,0,m->pict->width(),m->pict->height());
    }
   else {
   	QPainter*  p = new QPainter();
  	p->begin(this,true);
	QColor c; c.setRgb(255,255,255); p->setBrush(c);
  	p->drawRect(0,0,this->width(),this->height());
	p->end();
  	delete p;
	}
   
   }   
   
   
   void Graphique::refresh() {
   if(calcul_en_cours==1) setCursor(WaitCursor); 
   else {	   
	   if(type_affichage==1 || type_affichage==2) this->setCursor(CrossCursor);
	   else this->unsetCursor();
   }      
   
   if(m!=NULL) {    
	bitBlt((QWidget*)this,0,0,m->pict,0,0,m->pict->width(),m->pict->height());
    }
   else {
   	QPainter*  p = new QPainter();
  	p->begin(this,true);
	QColor c; c.setRgb(255,255,255); p->setBrush(c);
  	p->drawRect(0,0,this->width(),this->height());
	p->end();
  	delete p;
	}
   
   }   
   
   void Graphique::calcule(int i) {
   calcul_en_cours = i;
   }
