/***************************************************************************
                          model.h  -  description
                             -------------------
    begin                : mer fv 25 2004
    copyright            : (C) 2004 by Ferdy Jean-Baptiste
    email                : ferdy@univ-montp2.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef MODELE_H
#define MODELE_H

#include "xml.h"
#include "matrice.h"
#include "sfonction.h"
#include "trajectoire.h"
#include "valeur_propre.h"
#include "math.h"


//#include <stdio.h>
//#include <iostream.h>
//#include <stdlib.h>


   
 class isocline {
   public:
    isocline(int var_const,int var_dep,int l_par_nb,int l_par_nb,char** l_var_nom,char** l_par_nom,double* l_var,double* l_par,char* s);
    ~isocline();

    int   get_var_const();
    int   get_var_dep();
    char* get_var();
    fonction* get_y();
    void  ecrire(FILE* fichier,double xmax);
    
   private:    
    //char**    var;    
    int       var_const;  //variable dont la drive s'annule  
    int       var_dep;    //variable en fonction de laquelle y est calcul
    double*   x;      //pointeur vers la variable en fonction de laquelle y est calcule
    fonction* y;
   };       

 class modele {
   public:
   //constructeur, destructeur
      int ouvrir_modele(char* texte); //routine de base utilise par le constructeur
      modele(char* texte);            //construit un modle  partir d'une chaine de caractres
      modele(FILE* fichier);          //construit un modle  partir d'un fichier
      void detruit();                 //routine appele par le destructeur
      void detruit_var();             //destruction des variables
      void detruit_par();             //destruction des paramtres
      void detruit_var(int n);             //destruction des variables
      void detruit_par(int n);             //destruction des paramtres
      void detruit_var_par();         //destruction des variables et des paramtres
      ~modele();

   //routines de base
      int           valide();   //renvoit 1 si le modle est valide, i.e. s'il a au moins un paramtre, une variable et une fonction
      void          ecrire();   //crit un descriptif du modle
      char*         get_description();
      int           get_temps_continu();      
      
   //gestion des paramtres et des variables
      void          set_temps_continu(int i);                           //temps continu ou discret
      
      void          variable(double* v);                                //affecte les valeurs de (double*)v aux paramtres
      void          parametre(double* v);                               //affecte les valeurs de (double*)v aux variables
      void          variable(int i,double x);                           //affecte x  la variable i
      void          parametre(int i,double x);                          //affecte x au paramtre i
      
      void          ecrire_parametre();
      void          ecrire_variable();
      
      int           get_var_nb();
      int           get_par_nb();
      double        get_var(int i);
      double        get_par(int i);
      char*         get_var_nom(int i);
      char*         get_par_nom(int i);
      double*       get_var();
      double*       get_par();
      char**        get_var_nom();
      char**        get_par_nom();
      
      int           set_var_ini(char** s);
      void          get_var_ini(char*** s);
      void          set_var(int i,double x);
      void          set_par(int i,double x);
      void          set_var(double* x);
      void          set_par(double* x);

   //gestion des quilibres      
      int           get_eq_nb();                                        //renvoit le nombre d'quilibres
      fonction*     get_eq(int i);                                      //renvoit l'quilibre n i
      
      void          ecrire_equilibre();                                 //crit les valeurs numriques correspondant aux point d'quilibre dans la sortie standard
      void          ecrire_equilibre(FILE* fichier);                    //crit les valeurs numriques correspondant aux point d'quilibre dans un fichier      
      int           stable(fonction* eq);                               //teste la stabilit d'un quilibre
      int           stable(double* x);                                  //teste la stabilit d'un point
      
   //gestion des trajectoires
      int           get_tr_nb();       
      void          stop();						//stopppe le calcul en cours
      
      void          set_trajectoire(int i,trajectoire* t);              //remplace la trajectoire i par t
      trajectoire*  get_trajectoire(int i);                             //renvoie la trajectoire i
      
      void          suppr_trajectoire(int i);                           //supprime la trajectoire i
      void          suppr_trajectoire();                                //supprime toutes les trajectoires
      
      void          ajout_trajectoire(double tmax);                     //ajoute une trajectoire     
            
      void          rafraichir_trajectoire(int i,double tmax);          //rafraichir les trajectoires
      void          rafraichir_trajectoire(int i);  
      void          rafraichir_trajectoire(double tmax);
      void          rafraichir_trajectoire();                           
            
      void          ecrire_trajectoire(int i);                          //crit la trajectoire i
      void          ecrire_trajectoire();                               //crit toutes les trajectoires
      void          ecrire_trajectoire(FILE* fichier);                  //crit toutes les trajectoires dans un fichier
      void          ecrire_trajectoire_un_t(FILE* fichier);
      void          ecrire_trajectoire_sans_t(FILE* fichier);
      
  //gestion des isoclines      
      int           get_iso_nb();
      isocline*     get_iso(int i);

      void          ecrire_isocline(char* nom,double xmax,double ymax);

  //gestion de la jacobienne
      fonction**    get_jacob();
  
  //gestion des distributions 
     int            lire_distribution(char* tag,char* buf,int x_nb,char** x_nom,distribution** x_dist);
      
   private:    
    char*         description;      //description du modle, titre, nom

    int           temps_continu;    //temps continu (1, par dfaut) ou discret (0)    

    int           var_nb;           //nombre de variables
    int           par_nb;           //nbre de paramtres
    double        *var;             //valeurs des variables
    double        *par;             //valeurs des paramtres
    char          **var_nom;        //nom des variables
    char          **par_nom;        //nom des paramtres
    sfonction*    var_ini;          //fonction d'initialisation des variables

    sfonction*     f;               //systme d'quations

    int           eq_nb;            //nombre de points d'quilibre
    fonction**    eq;               //points d'quilibres

    int           iso_nb;           //nombre d'isoclines
    isocline**    iso;              //isoclines

    fonction**    jacob;            //matrice jacobienne

    int           tr_nb;            //nombre de trajectoires
    trajectoire** tr;               //trajectoires
   };
#endif
//fin
