/***************************************************************************
 *   Copyright (C) 2004 by Ferdy Jean-Baptiste                             *
 *   ferdy@univ-montp2.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nouveau_modele.h"
#include "form_nouveau_modele.h"

#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qtable.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qtoolbox.h>
nouveau_modele::nouveau_modele() : Form_nouveau_modele()
{
//connections aux slots
QObject::connect(pushButton_var_plus,SIGNAL(clicked()),this,SLOT(ajout_variable()));
QObject::connect(pushButton_var_moins,SIGNAL(clicked()),this,SLOT(suppr_variable()));
QObject::connect(pushButton_par_plus,SIGNAL(clicked()),this,SLOT(ajout_parametre()));
QObject::connect(pushButton_par_moins,SIGNAL(clicked()),this,SLOT(suppr_parametre()));
QObject::connect(pushButton_mat_plus,SIGNAL(clicked()),this,SLOT(ajout_matrice()));
QObject::connect(pushButton_mat_moins,SIGNAL(clicked()),this,SLOT(suppr_matrice()));
QObject::connect(pushButton_mat_up,SIGNAL(clicked()),this,SLOT(matrice_up()));
QObject::connect(pushButton_mat_down,SIGNAL(clicked()),this,SLOT(matrice_down()));

QObject::connect(pushButton_ok,SIGNAL(clicked()),this,SLOT(ok()));
QObject::connect(pushButton_annuler,SIGNAL(clicked()),this,SLOT(annuler()));

spinBox_taille->setMinValue(2);

//mise en forme des tables
int w = table_variable->width();
table_variable->setColumnStretchable(0,false);
table_variable->setColumnWidth(0,w/6);
table_variable->setColumnStretchable(1,false);
table_variable->setColumnWidth(1,w/6);
table_variable->setColumnStretchable(2,true);

table_parametre->setColumnStretchable(0,true);
table_parametre->setColumnStretchable(1,true);

table_matrice->setNumRows(0);
table_matrice->setColumnStretchable(0,true);
}

void nouveau_modele::ajout_variable()
{
int nb_var =  table_variable->numRows()+1;
table_variable->setNumRows(nb_var);
}

void nouveau_modele::ajout_parametre()
{
int nb_par =  table_parametre->numRows()+1;
table_parametre->setNumRows(nb_par);
}

void nouveau_modele::suppr_variable()
{
int pos = table_variable->currentRow();
table_variable->removeRow(pos);
}

void nouveau_modele::suppr_parametre()
{
int pos = table_parametre->currentRow();
table_parametre->removeRow(pos);
}

void nouveau_modele::ajout_matrice()
{
int n = spinBox_taille->text().toInt();
commande_matrice* dlg = new commande_matrice(n,n,this);

if(dlg->exec() == QDialog::Accepted) {
  n =  table_matrice->numRows()+1;
  table_matrice->setNumRows(n);
  table_matrice->setText(n-1,0,dlg->get_commande());
  }
}

void nouveau_modele::suppr_matrice()
{
int pos = table_matrice->currentRow();
table_matrice->removeRow(pos);
}

void nouveau_modele::matrice_up() {
int pos = table_matrice->currentRow();
if(pos>0) table_matrice->swapRows (pos,pos-1);
table_matrice->updateContents();
}

void nouveau_modele::matrice_down() {
int pos = table_matrice->currentRow();
if(pos<table_matrice->numRows()-1) table_matrice->swapRows (pos,pos+1);
table_matrice->updateContents();
}

void nouveau_modele::ok() {
if(valide()) accept();
else {
	QMessageBox::information( this, "KSimPop","Modle incomplet.");
	}
}

void nouveau_modele::annuler(){
reject();
}

int nouveau_modele::valide(){

switch(toolBox_modele->currentIndex())  {  
  case 0 : { //modle standard
     if(nb_var()<1) return(0);
     for(int i=0;i<nb_var();i++) {
       if(table_variable->text(i,0).isEmpty()) return(0);
       if(table_variable->text(i,1).isEmpty()) return(0);
       if(table_variable->text(i,2).isEmpty()) return(0);
       }
     break;
     }
  case 1 : { //modle matriciel 
     if(nb_var()<1) return(0);
     if(lineEdit_vecteur_nom->text().isEmpty()) return(0);
     if(lineEdit_vecteur_valeur->text().isEmpty()) return(0);
     if(lineEdit_matrice_defaut->text().isEmpty()) return(0);
     break;
     }
  }
return(1);
}

QString nouveau_modele::chaine() {
QString buf = "<modele>\n";
if(!lineEdit_nom->text().isEmpty()) {
		buf +="<nom>"+lineEdit_nom->text()+"</nom>\n";
		}

switch(toolBox_modele->currentIndex())  {
  case 0 : { //modle standard
	if(radioButton_temps_discret->isChecked()) {
		buf +="<temps>discret</temps>\n";
		}
	buf +="<variable>";
	for(int i=0;i<nb_var();i++) {
		if(!table_variable->text(i,0).isEmpty() && !table_variable->text(i,1).isEmpty() && !table_variable->text(i,2).isEmpty()) {
		  buf+=table_variable->text(i,0);
		  buf+="=";
		  buf+=table_variable->text(i,1);
		  buf+=" ";
		  }
		}
	buf +="</variable>\n";
	
	if(nb_par()>0) buf +="<parametre>";
	for(int i=0;i<nb_par();i++) {
		if(!table_parametre->text(i,0).isEmpty() && !table_parametre->text(i,1).isEmpty()) {
		  buf+=table_parametre->text(i,0);
		  buf+="=";
		  buf+=table_parametre->text(i,1);
		  buf+=" ";
		  }
		}
	if(nb_par()>0) buf +="</parametre>\n";
	
	buf +="<equation>\n";
	for(int i=0;i<nb_var();i++) {
		if(!table_variable->text(i,0).isEmpty() && !table_variable->text(i,1).isEmpty() && !table_variable->text(i,2).isEmpty()) {
		  buf+=table_variable->text(i,2);
		  buf+="\n";
		  }
		}
	buf +="</equation>\n";
	break;
	}
  case 1 : { //modle matriciel
        buf +="<variable> " + lineEdit_vecteur_nom->text() + "[" + spinBox_taille->text() + "] = " + lineEdit_vecteur_valeur->text() + " </variable>\n";
	if(nb_par()>0) buf +="<parametre>";
	for(int i=0;i<nb_par();i++) {
		if(!table_parametre->text(i,0).isEmpty() && !table_parametre->text(i,1).isEmpty()) {
		  buf+=table_parametre->text(i,0);
		  buf+="=";
		  buf+=table_parametre->text(i,1);
		  buf+=" ";
		  }
		}
	if(nb_par()>0) buf +="</parametre>\n";
	buf +="<equation>\n<matrice defaut=" + lineEdit_matrice_defaut->text() + ">\n";
	for(int i=0;i<table_matrice->numRows();i++) {
	 if(!table_matrice->text(i,0).isEmpty())  buf += table_matrice->text(i,0) + "\n";
	  }
	buf +="</matrice>\n</equation>\n";
	
    break;
    }
  }
buf += "</modele>\n";
return(buf);
}
void nouveau_modele::copie_variable(QTable* tab) 
{
int nb_var = table_variable->numRows();
tab->setNumRows(nb_var);
int i;
for(i=0;i<nb_var;i++) {
	tab->verticalHeader()->setLabel(i,table_variable->text(i,0));	
	tab->setText(i,0,table_variable->text(i,1));
	}
}

void nouveau_modele::copie_parametre(QTable* tab) 
{
int nb_par = table_parametre->numRows();
tab->setNumRows(nb_par);
int i;
for(i=0;i<nb_par;i++) {
	tab->verticalHeader()->setLabel(i,table_parametre->text(i,0));
	tab->setText(i,0,table_parametre->text(i,1));
	}
}

int nouveau_modele::nb_var()
{
switch(toolBox_modele->currentIndex())  {
  case 0 : { //modle standard
    return(table_variable->numRows());
    }
  case 1 :{ //modle matriciel 
    return(spinBox_taille->text().toInt());
    }
    }
}

int nouveau_modele::nb_par()
{
return(table_parametre->numRows());
}

//#include "nouveau_modele.moc"