/***************************************************************************
 *   Copyright (C) 2005 by Jean-Baptiste Ferdy                             *
 *   ferdy@univ-montp2.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
     
#include "objet.h"

//#define OBJET_MAX_CHAR 250

extern FILE*  FICHIER_SORTIE;

//----------------------------------------------------------------
//Routines pour la lecture et l'criture des objets

//constructeurs et destructeurs
objet::objet() {
  nom  = NULL;
  t  = 'n';
  lx = NULL;
  lo = NULL;
  x  = 0;
  };

objet::objet(char c,char* s) {
  if(s!=NULL && strlen(s)>0) {  
    nom  = (char*) malloc((strlen(s)+1)*sizeof(char));
    strcpy(nom,s);
    }
  else nom=NULL;
  t  = c;
  lx = NULL;
  lo = NULL;
  x  = 0;
  };

objet::objet(double z) {  
  t   = 'c';
  nom = NULL;
  lx  = NULL;
  lo  = NULL;
  x   = z;
  };
        
objet::~objet() {
  if(nom!=NULL) {free(nom); nom=NULL;}
  nom = NULL;
  t   = 'n';
  x   = 0;
  lx  = NULL;
  delete lo;
  lo  = NULL;
  };

int    objet::set_nom(char* s) {
  if(s==NULL || strlen(s)<=0) return(-1);
  if(nom!=NULL) free(nom);
  nom = (char*) malloc((strlen(s)+1)*sizeof(char));
  strcpy(nom,s);
  return(1);
  }
  
int    objet::is_var_dep(double* z) {
       if(t=='v' && lx==z) return(1);
       return(0);
       }
                     
//ecriture d'un objet dans un fichier
void objet::ecrire() {
  char* buf = this->tostring();
  fprintf(FICHIER_SORTIE,"%s",buf); 
  free(buf);
  }

//ecriture d'un objet dans une chaine de caractres
char* objet::concatene(char** s) {
  char* buf = tostring();
  char* s2  = ::concatene(s,buf);
  if(buf!=NULL) free(buf);  
  return(s2);
  }
  
char* objet::tostring() {          
  char* s = NULL;
  switch(t) {
    case 't' :                                        //template
    case 'f' :                                        //fonction
    case 'o' :                                        //oprateur    
    case 's' :                                        //chaine de caractres
    case 'v' : { s = ::concatene(&s,nom); break;}     //variable
    case 'c' : { s = ::concatene(&s,x); break;}       //constante
    default  : { s = ::concatene(&s,t); break;}
    }    
  return(s);  
  }
    
//recherche une variable ou un paramtre parmi une liste
int objet::cherche_x(int z_nb,char** z_nom) {
  if(nom==NULL || strlen(nom)==0) return(-1);
  for(int i=0;i<z_nb;i++) {
    if(z_nom[i]!=NULL && strcmp(nom,z_nom[i])==0) return(i);
    }
  return(-1);
  }
            
//lecture d'un objet
int objet::lire(char* s) {
  int   n,nbc;
  char* tmp;
  
  n=strlen(s); 
  if(n<=0) return(n);
  
  tmp = (char*) malloc((n+1)*sizeof(char));
  nbc=1; tmp[0]=s[0];
  switch(type_caractere(s[0]))  {
    //texte
    case 0 : {            
      while((type_caractere(s[nbc])==0 || type_caractere(s[nbc])==1) && (nbc)<n) {
        tmp[nbc]=s[nbc]; nbc++;
        }
      tmp[nbc]='\0';
      //l'objet est une fonction
      if(type(tmp)=='f') t='f';      
      else t='v';      
      set_nom(tmp);      
      break;
      }
    //numrique
    case 1 : {
      t='c';
      while(type_caractere(s[nbc])==1 && nbc<n) {
        tmp[nbc]=s[nbc]; nbc++;
        }
      tmp[nbc]='\0';
      x=atof(tmp);
      set_nom("const");      
      break;
      }
    //oprateur
    case 2 : {
      t='o';      
      tmp[0]=s[0]; tmp[1]='\0';
      set_nom(tmp);      
      break;
      }
    //sparateur
    case 3 : {
      t=s[0];      
      tmp[0]=s[0]; tmp[1]='\0';
      set_nom(tmp);      
      break;
      }
    //template
    case 4 : {
      t='t';
      while(type_caractere(s[nbc])!=4 && nbc<n) {
        tmp[nbc]=s[nbc]; nbc++;
        }      
      tmp[nbc]=s[nbc];
      nbc++;
      tmp[nbc]='\0';
      set_nom(tmp);
      break;
      }
    //indices
    case 5 : {
      t='i';
      while(type_caractere(s[nbc])!=5 && nbc<n) {
        tmp[nbc]=s[nbc]; nbc++;
        }      
      tmp[nbc]=s[nbc];
      nbc++;
      tmp[nbc]='\0';
      set_nom(tmp);
      break;
      }
    }
  
  free(tmp);
  return(nbc);      
  }
      
//valutation d'un objet
double objet::eval() {
  switch(t) {
    case 'v': return(*lx);
    case 'c': return(x);  
    default:  {fprintf(FICHIER_SORTIE,"ERREUR objet::eval - impossible d'valuer l'objet\n"); return(0);}
    }
  }
  
int objet::construit_indice(char*** nom_i,int** val_i,int** val_max,int** val_min) {
  if(t!='t') return(0);
  int n = strlen(nom);
  int cmpt = 0;
  *nom_i   = (char**) malloc(sizeof(char*));
  *val_i   = (int*) malloc(sizeof(int));
  *val_max = (int*) malloc(sizeof(int));
  *val_min = (int*) malloc(sizeof(int));
  
  char* buf = (char*) malloc((n+1)*sizeof(char));
  
  for(int i=0;i<n;i++) {
    if(nom[i]=='&' && i+1<n) {
      i++;      
      
      //lecture du nom
      int k=0;
      while(i+k<n && nom[i+k]!='[' && nom[i+k]!=']' && type_caractere(nom[i+k])==0) {buf[k]=nom[i+k]; k++;} buf[k]='\0';
      
      //le nom existe-t-il dj ?
      k=0;
      for(k=0;k<cmpt && strcmp(buf,(*nom_i)[k])==0;k++); 
      //le nom n'existe pas
      if(k==cmpt) {
        if(cmpt>0) {
           *nom_i   = (char**) realloc(*nom_i,(cmpt+1)*sizeof(char*));
	   *val_i   = (int*)   realloc(*val_i,(cmpt+1)*sizeof(int));
	   *val_min = (int*)   realloc(*val_min,(cmpt+1)*sizeof(int));
	   *val_max = (int*)   realloc(*val_max,(cmpt+1)*sizeof(int));
	   }
	(*nom_i)[cmpt] = (char*) malloc((strlen(buf)+1)*sizeof(char));
	strcpy((*nom_i)[cmpt],buf);
	(*val_i)[cmpt]   = -1;
	(*val_min)[cmpt] = -1;
	(*val_max)[cmpt] = -1;
	cmpt++;
	}
      }
    }
  if(buf!=NULL) free(buf);
  return(cmpt-1);
  }
    
//valutation d'une liste de valeurs d'indice
int objet::eval(int var_nb,int nb_i,char** nom_i,int* val_min,int* val_max) {
  if(t!='i') return(0);
  char c = parenthese(nom);
  if(c!='0') {
       fprintf(FICHIER_SORTIE,"ERREUR objet::eval - %c manquant dans %s\n",c,nom);
       return(-1);
      }
  
  int n = strlen(nom);        
  char* buf = (char*) malloc((n+1)*sizeof(char));
       
  //lecture du nom
  int i = 0; 
  int j = 0;
  while(i<n && type_caractere(nom[i])!=0) i++;     
  while(i+j<n && type_caractere(nom[i+j])==0) {buf[j]=nom[i+j]; j++;} buf[i]='\0';    
  i+=j;
  //recherche du nom parmis les indices dtects 
  j = 0; 
  while(j<nb_i && nom_i[j]!=NULL) {
    if(strcmp(nom_i[j],buf)==0) {  
      fprintf(FICHIER_SORTIE,"ATTENTION objet::eval - dclar multiple de l'indice %s !\n",buf);
      free(buf);
      return(0);
      }
    j++;
    }
  nom_i[j] = (char*) malloc((strlen(buf)+1)*sizeof(char));
  strcpy(nom_i[j],buf);free(buf);
    
  //lecture borne infrieure
  while(nom[i]!='-' && type_caractere(nom[i])!=1) i++;
  val_min[j] = atoi(&(nom[i]));  
  if(val_min[j]<0) val_min[j] = val_min[j]+1+var_nb; 
  i++; 
  //lecture borne suprieure
  while(nom[i]!='-' && type_caractere(nom[i])!=1) i++;
  val_max[j] = atoi(&(nom[i]));
  if(val_max[j]<0) val_max[j] = val_max[j]+1+var_nb;
  
  if(val_max[j]<val_min[j]) {
    i = val_max[j];
    val_max[j] = val_min[j];
    val_min[j] = i;
    }
  
     
  return(1+val_max[j]-val_min[j]);
  }    

//******************************************************************************
//                   TRAITEMENT DES CHAINES D'OBJETS                          
//******************************************************************************

//transforme une chaine de caractres en une liste d'objets    
int lire_chaine_objet(char* s,objet*** chaine_objet,int var_nb,char** var_nom,double* var,int par_nb,char** par_nom,double* par) {
    int i,j,k,n;

    char c = parenthese(s);
    if(c!='0') {
       fprintf(FICHIER_SORTIE,"ERREUR lire_chaine_objet - %c manquant dans %s\n",c,s);
       return(-1);
      }      
    n=strlen(s);
      
    for(i=0,j=0;i<n;j++) {    
      while((s[i]==' ' || s[i]=='\t' || s[i]=='\0' || s[i]=='\n') && i<n) i++;    
      if(j>0) {
        (*chaine_objet) = (objet**) realloc(*chaine_objet,(j+1)*sizeof(objet*));
	}
      else {
        (*chaine_objet) = (objet**) malloc(sizeof(objet*));
	}
      (*chaine_objet)[j] = new objet();
      i+=((*chaine_objet)[j])->lire(&(s[i]));

      //recherche des variables et paramtres
      if((*chaine_objet)[j]->get_t()=='v') {
        k = (*chaine_objet)[j]->cherche_x(var_nb,var_nom);
        if(k>=0) {
          (*chaine_objet)[j]->set_lx(&(var[k]));
          }
        else {
          k = (*chaine_objet)[j]->cherche_x(par_nb,par_nom);
          if(k>=0) {
            (*chaine_objet)[j]->set_lx(&(par[k]));
            }
          else {
            fprintf(FICHIER_SORTIE,"ERREUR fonction::lire - variable ou paramtre %s inconnu\n",(*chaine_objet)[j]->get_nom());
            return(-1);
            }
          }
        }
      }
  return(j);
  }

//dtruit les objets qui ne seront pas conservs en tant que tel dans l'opration
void detruit_chaine_objet(int n,objet** chaine_objet) {
  for(int i=0;i<n;i++) {
    char c = chaine_objet[i]->get_t();
    if(c!='v' && c!='c') {
        delete chaine_objet[i];
      }
    }
}
      
//calcule le nombre d'arguments passs  une fonction
//les arguments sont compris entre deux parenthses et spars par des virgules
int nombre_arguments(objet** chaine,int n) {
  if(chaine[0]->get_t()!='(') {fprintf(FICHIER_SORTIE,"ERREUR nombre_arguments - ( manquante\n"); return(-1);}
  
  int cmpt=0;
  int i=1;
  int nb;
  
  while(chaine[i]->get_t()!=')' && i<n) {
    int j=0;
    nb=0; //nb parenthses ouvertes non fermes    
    while(i+j<n && (nb!=0 || (chaine[i+j]->get_t()!=',' && chaine[i+j]->get_t()!=')'))) {
      if(chaine[i+j]->get_t()==')') nb--;
      if(chaine[i+j]->get_t()=='(') nb++;
      j++;
      }
    i+=j;
    //if((chaine[i]->get_t()==')' || chaine[i]->get_t()==',') && j>0) cmpt++;      
    if((chaine[i]->get_t()==')' || chaine[i]->get_t()==',') && (chaine[i-1]->get_t()!='(' && chaine[i-1]->get_t()!=',') && j>0) cmpt++;      
    if(chaine[i]->get_t()==',') i++;
    }
  if(i>1 && nb!=0) {fprintf(FICHIER_SORTIE,"ERREUR nombre_arguments - ) manquante\n"); return(-1);} //la parenthse fermante n'a pas t trouve
  
  return(cmpt);
  }

//renvoit la position de la parenthse fermante dans une chaine d'objet qui commence par une parenthse ouvrante    
int parenthese_fermante(objet** chaine,int n) {
  if(chaine[0]->get_t()!='(') return(-1);
  int i=1;
  int cmpt=1;
  while(cmpt!=0 && i<n) {
    if(chaine[i]->get_t()==')') cmpt--;
    if(chaine[i]->get_t()=='(') cmpt++;
    if(cmpt!=0) i++;
    }
  if(chaine[i]->get_t()!=')') return(-1);
  return(i);
  }

//renvoit le nombre d'objets qui dfinissent le nme argument dans une chaine de nb objets
//renvoit -1 si l'argument n'est pas trouv    
int longueur_argument(objet** chaine,int n,int nb) {
  if(n<0 || (chaine[0]->get_t()!='(' && chaine[0]->get_t()!=',')) return(-1);
  //calcul du dbut de l'argument  
  int i=1;
  int cmpt=0; //compte les arguments jusqu' n
  while(cmpt<n && i<nb) {
    if(chaine[i]->get_t()!=',') cmpt++;
    i++;
    }
  if(cmpt<n) return(-1); //pas assez d'arguments
  //calcul du dbut de l'argument
  cmpt = 0; //compte les parenthses ouvertes non fermes sinon on s'arrte  la 1re ) rencontre
  while(i<nb && ((chaine[i]->get_t()!=',' && chaine[i]->get_t()!=')') || cmpt!=0)) {
    if(chaine[i]->get_t()==')') cmpt--;
    if(chaine[i]->get_t()=='(') cmpt++;
    i++;
    }
  if(chaine[i]->get_t()!=')' && chaine[i]->get_t()!=',') return(-1);
  return(i);
  }
  
//affichage des erreurs de syntaxe dtectes pendant la compilation
void objet::erreur_syntaxe() {
  fprintf(FICHIER_SORTIE,"ERREUR compile");  
  switch(t) {
    case 'o': {fprintf(FICHIER_SORTIE," - oprande  manquant pour %s",get_nom()); break;}
    case 'f': {fprintf(FICHIER_SORTIE," - nombre d'arguments incorrect pour la fonction %s",get_nom()); break;}
    case 'v': {fprintf(FICHIER_SORTIE," - oprateur manquant pour %s",get_nom()); break;}
    case 'c': {fprintf(FICHIER_SORTIE," - oprateur manquant pour %g",get_x()); break;}
    default : {fprintf(FICHIER_SORTIE," - erreur dans la compilation de la fonction %s",get_nom()); break;}
    }
  fprintf(FICHIER_SORTIE,"\n");
  }
