/***************************************************************************
 *   Copyright (C) 2005 by Jean-Baptiste Ferdy                             *
 *   ferdy@univ-montp2.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 #include "operation_base.h"
 
extern FILE*  FICHIER_SORTIE;
extern double ODE_PRECISION;
extern double MAX_FLOAT;        //valeur max des variables permise (lue dans un fichier)
extern int    MAX_TEMPS_CALCUL;
extern int    PROBLEME_CALCUL;
extern char   MSG_PROBLEME_CALCUL[];

//**********************************************************************************
//constructeurs et destructeur
//**********************************************************************************
operationBase::operationBase() {
  t='n';
  n=2;
  ob = (objet**) malloc(n*sizeof(objet*));     
  x  = (double*) malloc(n*sizeof(double));
  for(int i =0;i<n;i++) {ob[i]=NULL; x[i]=0.0;}
  f=NULL;
  }
  
operationBase::operationBase(int nb) {
  t='n';
  n=nb;
  ob = (objet**) malloc(n*sizeof(objet*));     
  x = (double*) malloc(n*sizeof(double));
  for(int i=0;i<n;i++) {ob[i]=NULL; x[i]=0.0;}  
  f=NULL;
  }
  
operationBase::operationBase(int nb,char c) {  
  n=nb;
  t=c;  
  f = get_fonction_evaluation(c);
  if(f==NULL) {t='n'; nb=0;}
  else {
    ob = (objet**) malloc(n*sizeof(objet*));       
    x  = (double*) malloc(n*sizeof(double));
    for(int i=0;i<n;i++) {ob[i]=NULL; x[i]=0.0;}    
    }  
  }

operationBase::~operationBase() {  
  //dans le cas o la fonction n'a pas d'arguments (par exemple dans somme), x est en fait un pointeur vers var 
  //et il ne faut pas le deleter !
  for(int i=0;i<n && ob!=NULL;i++) {
    if(ob[i]!=NULL) {
      if(x!=NULL) {free(x); x=NULL;} 
      delete ob[i]; ob[i]=NULL;
      }
    }
  if(ob!=NULL) {free(ob); ob=NULL;}
  f=NULL; n=0; t='n';
  }

//*************************************************************************************
//routines de base 
//seule les routines qui sont rimplmentes dans les classe drive sont dcrites ici
//*************************************************************************************
  
//retourne le nombre d'oprandes d'une opration  
int operationBase::get_nb_op(){  
  int cmpt=0;
  for(int i=0;i<n;i++) if(ob[i]!=NULL) cmpt++;
  return(cmpt);
  };
  
//l'opration dpend-elle de la variable x ?
int operationBase::is_var_dep(double* xx) {
  for(int i=0;i<n;i++) {
	if(&(x[i]) == xx) return(1);
	if(ob[i]!=NULL && ob[i]->is_var_dep(xx) == 1) return(1);
	}  
  //retour par dfaut
  return(0);
  }

//change le nombre d'arguments
void operationBase::set_n(int i) {
  if(n>0) {    
    ob = (objet**) realloc(ob,i*sizeof(objet*));  
    x  = (double*) realloc(x,i*sizeof(double));
    }
  else {    
    ob = (objet**) malloc(i*sizeof(objet*));  
    x  = (double*) malloc(i*sizeof(double));
    }
  
  //si le nombre d'oprateurs a augment, on initialise les oprateurs suplmentaires
  for(int j=n;j<i;j++) {ob[j]=NULL; x[j]=0.0;}
  n = i;
  }
    
//affecte l'objet xx comme ime argument
void operationBase::reset_ob(int i,objet* xx){
  if(i<n) ob[i]=xx;
  else {
    fprintf(FICHIER_SORTIE,"ERREUR operationBase::reset_ob - l'objet #%d n'existe pas !",i);
    }
  }
              
//routines d'valuation de l'opration 
int operationBase::eval_argument() {
  //si le pointeur vers la fonction d'valuation n'est pas valide, on retourne le premier argument
  if(t=='n' || f==NULL) {
    if(n>0) {      
      if(ob[0]!=NULL) x[0] = ob[0]->eval();
      return(1);      
      }
    fprintf(FICHIER_SORTIE,"ERREUR operationBase:eval_argument - impossible d'valuer une fonction vide !\n",t); 
    return(-1);
    }
  
  if(get_nb_op() < ::nb_arg_min(t) || get_nb_op() > ::nb_arg_max(t)){
    fprintf(FICHIER_SORTIE,"ERREUR operationBase:eval_argument - la fonction %s ne peut tre value avec %d arguments!\n", ::nom_fonction(t),n); 
    return(-1);
    }
  
  //calcul des arguments
  for(int i=0;i<n;i++) {    
    if(ob[i]!=NULL)  x[i]=ob[i]->eval();        
    }
  return(1);
  }         
        
double operationBase::eval()             //pas de test de validit du calcul
  {  
  int test = 1;
  double z = operationBase::eval(&test);
  return(z);
  }

double operationBase::eval(int* test)   //si pas d'erreurs de calcul test=1 sinon test=-1
  {    

  //des erreurs de calculs ont t commise dans la fonction appelante
  if(*test == -1) return(0.0);
  *test = eval_argument();
  if(*test == -1) return(0.0);
  
  //fonction vide sans erreur
  if(t=='n' || f==NULL) return(x[0]);    
  
  //valuation
  double z = (*f)(n,x,test);
  
  //teste la validit du rsultat  
  if(!gsl_finite(z)) {    
    *test=-1;
    sprintf(MSG_PROBLEME_CALCUL,"ERREUR operationBase:eval - valeur infinie!\n");
    fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
    }
  
  //si pas de problme de calcul renvoit le rsultat
  if((*test)==1) return(z);   
  
  //sinon renvoit zro
  sprintf(MSG_PROBLEME_CALCUL,"ERREUR operationBase:eval - erreur d'valuation pour la fonction %s\n",nom_fonction(t));
  fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
  PROBLEME_CALCUL = -1; 
  return(0);  
  }
  
//routines d'criture
void operationBase::ecrire() {  
  char* buf = this->tostring();
  fprintf(FICHIER_SORTIE,"%s",buf);  
  free(buf);
}

char* operationBase::concatene(char** s) {
  char* buf = tostring();
  char* s2  = ::concatene(s,buf);
  free(buf);
  return(s2);
  }
  
char* operationBase::tostring() {  
  char* s = NULL; 
    
  switch(::type(t)) {
    case 'o': {        
	if(ob[0]!=NULL) {	
	  if(!((t=='+' || t=='-') && ob[0]->get_t()=='c' && ob[0]->eval()==0.0)) {
	    s = ob[0]->concatene(&s);
	    }
	  }
	s = ::concatene(&s,t);
	if(ob[1]!=NULL) s= ob[1]->concatene(&s);
	break;
	}
    case 'f': {
        s = ::concatene(&s,nom_fonction(t));
	s = ::concatene(&s,'(');
        for(int i=0;i<n;i++) {
          if(ob[i]!=NULL) s = ob[i]->concatene(&s);
	  if(i<n-1 && (ob[i]!=NULL)) s = ::concatene(&s,',');
	  }
        s = ::concatene(&s,')');
        break;
      }           
   default :  {
      if(ob[0]!=NULL) s = ob[0]->concatene(&s);      
      break;
      }
   }   
   return(s);
}

//
