/***************************************************************************
 *   Copyright (C) 2005 by Jean-Baptiste Ferdy                             *
 *   ferdy@univ-montp2.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 #ifndef OPERATION_BASE_H
 #define OPERATION_BASE_H

 #include "objet.h" 
 #include "parseur.h"
 #include "specification_fonction.h"
 #include "specification_distribution.h"
 
 #include <gsl/gsl_math.h>
 
 class operationBase {
  public:
    //constructeurs
    operationBase();                  //opration vide n=2
    operationBase(int nb);            //opration vide n=nb
    operationBase(int nb,char c);     //opration de type c  nb arguments. La fonction d'valuation est affecte.
    
    //destructeur
    virtual ~operationBase();            
            
    //oprations sur l'identifiant t
    char   get_t()                            //identifiant opration     
       {return(t);};
    void   set_t(char x)                      //fixe l'entifiant de l'opration
       {t = x;
        if(::nb_arg_min(x)==::nb_arg_max(x)) n = ::nb_arg_min(x);
        f = get_fonction_evaluation(x);
       };
    char   type()                             //oprateur ou fonction ?  
       {return(::type(t));}; 
    int    nb_arg_min()                       //nombre minimum d'oprandes ou d'arguments
       {return(::nb_arg_min(t));};
            
    //nombe d'aguments affects   
    virtual int    get_nb_op();               //nombres d'arguments affects
    int    get_nb_op_manquant()               //nombre minimum d'arguments  affecter pour rendre l'opration valide
       {return(n-get_nb_op());};
    int    valide()                           //l'opration est-elle valide ?
       //{if(n<=0 || (t!='n' && get_nb_op_manquant()>0)) return(-1); return(1);}
       {if(t!='n' && get_nb_op_manquant()>0) return(-1); return(1);}              
    //l'opration dpend-elle de la variable x ?         
    virtual int    is_var_dep(double* xx);        
    
    //dimention des tableaux d'arguments
    virtual void set_n(int i);
    
    int get_n()
      {return(n);};
    
    //arguments
    void set_x(double* xx)                   //affecte un vecteur xx au tableau de valeurs d'argument x
      {x=xx;};
    
    double* get_x()
      {return(x);}
      
    void   reset_ob(int i,objet* xx);       //affecte xx au ime objet du tableau d'arguments
    
    objet* get_ob(int i) 
      {if(i>-1&&i<n) return(ob[i]); return(NULL);};
    
    //affecte la fonction d'valuation
    void set_f(func_pt xf) 
      {f=xf;};
      
    func_pt get_f() 
      {return(f);};
            
    //routines d'valuation de l'opration                   
    virtual int eval_argument();            //value les arguments 
    double  eval();                         //pas de test de validit du calcul
    double  eval(int* test);                //si pas d'erreurs de calcul test=1 sinon test=-1

    //routine d'criture
    void    ecrire();
    char*   concatene(char** s);
    virtual char* tostring();
    
  protected:
    char        t;     //caractre identifiant la fonction ou l'oprateur
    int         n;     //nombre d'arguments
    double     *x;     //tableau des valeurs d'argument
    objet     **ob;    //tableau des arguments de type objet
    func_pt     f;     //pointeur vers la fonction d'valuation
  };      
  
  #endif
//
