/***************************************************************************
 *   Copyright (C) 2005 by Jean-Baptiste Ferdy                             *
 *   ferdy@univ-montp2.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "operation_simple.h"
 
extern FILE*  FICHIER_SORTIE;
extern double ODE_PRECISION;
extern double MAX_FLOAT;
extern int    MAX_TEMPS_CALCUL;
extern int    PROBLEME_CALCUL;
extern char   MSG_PROBLEME_CALCUL[];

//routines pour la lecture et l'criture des oprations-------------------------------------------
//constructeur et destructeur de base
operationSimple::operationSimple() : operationBase() {  
  op = (operationSimple**) malloc(n*sizeof(operationSimple*));     
  for(int i =0;i<n;i++) {op[i]=NULL;}
  pere=NULL;  
  }
  
operationSimple::operationSimple(int nb) : operationBase(nb) {
  op = (operationSimple**) malloc(n*sizeof(operationSimple*));      
  for(int i =0;i<n;i++) {op[i]=NULL;}
  pere=NULL;
  }
  
operationSimple::operationSimple(int nb,char c) : operationBase(nb,c) {  
  op = (operationSimple**) malloc(n*sizeof(operationSimple*));     
  for(int i =0;i<n;i++) {op[i]=NULL;}
  pere=NULL;
  }

void operationSimple::cp(operationSimple* o) {
  set_n(o->get_n());
  t    = o->get_t();
  pere = o->get_pere();
  f    = o->get_f();
  x    = o->get_x();
  for(int i=0;i<n;i++) {
    ob[i] = o->get_ob(i);
    op[i] = o->get_op(i);
    }
  }
    
operationSimple::~operationSimple() {  
  for(int i=0;i<n && op!=NULL;i++) {if(op[i]!=NULL) {if(x!=NULL) {free(x); x=NULL;} delete op[i];}} 
  if(op!=NULL) {free(op); op=NULL;}
  pere=NULL;
  f=NULL;
  }
  
//************************************************************************
// routine hrites de la classe operation_base mais redfinies
//************************************************************************  
  //nombres d'arguments affects  
int operationSimple::get_nb_op() {  
  int cmpt=0;
  for(int i=0;i<n;i++) if(ob[i]!=NULL || op[i]!=NULL) cmpt++;
  return(cmpt);
  };  
                     
    //l'opration dpend-elle de la variable x ?         
int operationSimple::is_var_dep(double* xx) {
  for(int i=0;i<n;i++) {
	if(&(x[i]) == xx) return(1);
	if(ob[i]!=NULL && ob[i]->is_var_dep(xx) == 1) return(1);
	if(op[i]!=NULL && op[i]->is_var_dep(xx) == 1) return(1);
	}  
  //retour par dfaut
  return(0);
  }
    
    //redimentionne les tableaux d'arguments
void operationSimple::set_n(int i) {  
  if(i<=0) {
    if(op!=NULL) free(op); if(ob!=NULL) free(ob); if(x!=NULL) free(x);
    op=NULL; ob=NULL; x=NULL;
    n=0;
    return;
    }
    
  if(op!=NULL)  op = (operationSimple**) realloc(op,i*sizeof(operationSimple*));  
  else          op = (operationSimple**) malloc(i*sizeof(operationSimple*));
  if(ob!=NULL)  ob = (objet**) realloc(ob,i*sizeof(objet*));  
  else          ob = (objet**) malloc(i*sizeof(objet*));
  if(x!=NULL)   x  = (double*) realloc(x,i*sizeof(double));
  else          x  = (double*) malloc(i*sizeof(double));
  
  //si le nombre d'oprateurs a augment, on initialise les oprateurs suplmentaires
  for(int j=n;j<i;j++) {op[j]=NULL; ob[j]=NULL; x[j]=0.0;}
  n = i;
  }
                    
    //routines d'valuation de l'opration                   
int operationSimple::eval_argument() {
  //si le pointeur vers la fonction d'valuation n'est pas valide, on retourne le premier argument
  if(t=='n' || f==NULL) {
    if(n>0) {      
      if(ob[0]!=NULL) x[0] = ob[0]->eval();
      if(op[0]!=NULL) x[0] = op[0]->eval();
      return(1);      
      }
    fprintf(FICHIER_SORTIE,"ERREUR operationSimple:eval_argument - impossible d'valuer une fonction vide !\n",t); 
    return(-1);
    }
  
  if(get_nb_op() < ::nb_arg_min(t) || get_nb_op() > ::nb_arg_max(t)){
    fprintf(FICHIER_SORTIE,"ERREUR operationBase:eval_argument - la fonction %s ne peut tre value avec %d arguments!\n", nom_fonction(t),n); 
    return(-1);
    }
        
  //calcul des arguments
  for(int i=0;i<n;i++) {    
    if(ob[i]!=NULL)  x[i]=ob[i]->eval();        
    if(op[i]!=NULL)  x[i]=op[i]->eval(); 
    }
  return(1);
  }   

//routines d'criture  
char* operationSimple::tostring() {  
  char* s = NULL; 
    
  switch(::type(t)) {
    case 'o': {        
	if(ob[0]!=NULL) {	
	  if(!((t=='+' || t=='-') && ob[0]->get_t()=='c' && ob[0]->eval()==0.0)) {
	    s = ob[0]->concatene(&s);
	    }
	  }
	if(op[0]!=NULL) {
	  char c = op[0]->t;
	  if(::type(c)!='f' && (t=='/' || ((t=='*'||t=='^') && (c=='+' || c == '-')))) s=::concatene(&s,'(');
	  s = op[0]->concatene(&s);
	  if(::type(c)!='f' && (t=='/' || ((t=='*'||t=='^') && (c=='+' || c == '-')))) s=::concatene(&s,')');
	  }
	s = ::concatene(&s,t);

	if(ob[1]!=NULL) s = ob[1]->concatene(&s);

	if(op[1]!=NULL) {
	  char c = op[1]->t;
	  if(::type(c)!='f' && (t=='/' || ((t=='*'||t=='^') && (c=='+' || c == '-')))) s=::concatene(&s,'(');
	  s = op[1]->concatene(&s);
	  if(::type(c)!='f' && (t=='/' || ((t=='*'||t=='^') && (c=='+' || c == '-')))) s=::concatene(&s,')');
	  }
	break;
	}
    case 'f': {
        s = ::concatene(&s,nom_fonction(t));
	s = ::concatene(&s,'(');
        for(int i=0;i<n;i++) {
          if(ob[i]!=NULL) s = ob[i]->concatene(&s);
	  if(op[i]!=NULL) s = op[i]->concatene(&s);	      
	  if(i<n-1 && (ob[i]!=NULL || op[i]!=NULL)) s = ::concatene(&s,',');
	  }
        s = ::concatene(&s,')');
        break;
      }           
   default :  {
      if(ob[0]!=NULL) s = ob[0]->concatene(&s);        
      if(op[0]!=NULL) s = op[0]->concatene(&s);         
      break;
      }
   }   
   return(s);
}
      
//************************************************************************
//routines non hrites
//************************************************************************    

//constructeur compilateur************************************************
operationSimple::operationSimple(objet** chaine,int nb,int var_nb,char** var_nom,double* var,int par_nb,char** par_nom,double* par) {  
  
  //affectation des membres de l'opration courantes  
  x    = NULL;
  ob   = NULL;
  op   = NULL;
  pere = NULL;
  n    = 0;
  t    = 'n';
  f    = NULL;
  
  
  //compilation de la chaine d'objets
  int             i,nbop;
  operationSimple *courant,*haut,*tmp;
  
  int pos=0;
  courant  = new operationSimple();
  haut     = courant;
  
  while(pos<nb) {          
    if(pos < nb) {    
      switch(chaine[pos]->get_t()) {
    
//l'objet courant est un oprande---------------------------------------------------
        //parenthse ouvrante: on compile jusqu' la parenthse fermante pour obtenir l'oprande
        case '(' : {                          
	  i = parenthese_fermante(&(chaine[pos]),nb-pos);  
	  pos++;
	  i--;
          tmp = new operationSimple(&(chaine[pos]),i,var_nb,var_nom,var,par_nb,par_nom,par);
          if(tmp->valide()==-1) {
	    delete tmp;
	    return;
	    }
          //tmp->set_pere(courant);
          pos = pos+i;
          if(courant->set_o(tmp)==-1) {
	     chaine[pos]->erreur_syntaxe(); 
	     return;
	     };                          
          break;
          }
        //'oprande est une chaine de caractres, une variable ou un paramtre
        case 's' :
	case 'c' :
        case 'v' : {              
          if(courant->set_o(chaine[pos])==-1) {
	    chaine[pos]->erreur_syntaxe();
	    return;
	    };          
          break;
          }
        
//l'objet courant est un oprateur--------------------------------------------------
        case 'o' : {
          //type d'oprateur (premire lettre du nom)
	  char c = chaine[pos]->get_nom()[0]; 
	  
	  //nombre d'oprande de l'opration courante
	  nbop = courant->get_nb_op();        

          //on est  la fin de la chaine d'objets et l'opration  construire a besoin d'oprandes
	  //erreur de syntaxe
	  if(::nb_arg_min(c)>0 && pos==nb-1) {
	      chaine[pos]->erreur_syntaxe(); 
	      return;
	      }
	      
     //*****l'opration courante n'a pas d'oprateur dfini : on lui affecte l'oprateur qui vient d'tre lu     
          if(courant->get_t()=='n') {
            if(nbop==0) {   //aucun oprande
              if(c=='*' || c=='/') {
	        chaine[pos]->erreur_syntaxe(); 
		return;
		}
              if(c=='-' || c=='+') courant->set_o(0.0);       
	      //if(c=='-' || c=='+') courant->set_n(1); //a devrait marcher, mais non...
              }    
	    courant->set_t(c);
            }            
          
     //*****l'opration courante a dj un oprateur de dfini
          else {            
	    //vrification que l'opration prcdente a le bon nombre d'oprandes
             if(nbop < courant->nb_arg_min()) {
	         chaine[pos]->erreur_syntaxe(); 
		 return;
		 } 
              
            //construction de la nouvelle opration courante
	    tmp = new operationSimple(2,c);
            
	    //traitement des priorit entre oprateurs
	    switch(c) {
              case '*' :
              case '/' : {
            //***** * et / s'appliquent au dernier oprande
                //si l'opration courante est une fraction ou une puissance, on va au pre
                if(courant->get_t() == '/' || courant->get_t() == '^')  {                 
                  if(courant->get_pere()!=NULL) courant = courant->get_pere();
                  }

                //tant que l'opration courante est une fonction, on va au pre
                while(courant->get_pere()!=NULL && (courant->type()!='o')) {
                      courant = courant->get_pere();
                      }
                      
                //on est  la racine et on ne peut pas appliquer l'oprateur au 2me oprande...
                //on place l'opration nouvellement cre au sommet de l'arbre
                char cc = courant->get_t();
                if(courant==haut && (::type(cc)!='o' || cc=='/' || cc=='^')) {
                    tmp->set_o(haut);
                    courant=tmp;
                    haut=tmp;
                    }

                //l'oprateur s'applique au 2me oprande
                else {                                                                               
                    tmp->set_o(courant->get_ob(1));
                    tmp->set_o(courant->get_op(1));
                    //get_ob et get_op renvoient NULL si l'objet ou l'opration ne sont pas affects
                    //tmp ne reoit donc qu'un objet ou opration
                    courant->reset_o(tmp);                    
                    courant = tmp;
                  }
                break;
                }
            //***** ^ s'applique au dernier oprande
              case '^' : {
                //si l'opration courante n'est pas une fonction...
                while(courant->get_pere()!=NULL && courant->type()!='f') {
                      courant = courant->get_pere();
                     }                      
                tmp->set_o(courant->get_ob(1));
                tmp->set_o(courant->get_op(1));
                courant->reset_o(tmp);                
                courant = tmp;                
                break;
                }
            //***** + et - s'appliquent  tous les oprandes qui prcdent
              case '+':
              case '-': {
                tmp->set_o(haut);
                haut    = tmp;
                courant = tmp;
                break;
                }
            //***** opateur inconnu             
	     default : {
	        fprintf(FICHIER_SORTIE,"ERREUR compile - oprateur %s inconnue\n",chaine[pos]->get_nom());
	        return;
	        };      
              }
            }//fin du switch sur le type d'oprateur
          break;
          }//fin du case 'o'
//l'objet courant est une fonction---------------------------------------------------
        case 'f' : {
          //dtermine le type de fonction
	  char c = symbole_fonction(chaine[pos]->get_nom()); 
	  if(c=='?') {
	    fprintf(FICHIER_SORTIE,"ERREUR compile - fonction %s inconnue\n",chaine[pos]->get_nom());
	    return;
	    };
	  //calcul du nombre d'arguments
	  pos++; //on se place sur la parenthse ouvrante qui suit la fonction
	  nbop = nombre_arguments(&(chaine[pos]),nb-pos);
	  if(::nb_arg_correct(c,nbop)!=1) {
	    chaine[pos-1]->erreur_syntaxe(); return;
	    }
	  //construction de la fonction
	  tmp = new operationSimple(nbop,c);          
          for(int i=0;i<nbop;i++) {
	  int j = longueur_argument(&(chaine[pos]),0,nb-pos);
          if(j>0) {
	    operationSimple* arg = new operationSimple(&(chaine[pos+1]),j-1,var_nb,var_nom,var,par_nb,par_nom,par);
	    if(!arg) {
	      return;
	      }
	    tmp->set_o(i,arg);
	    pos+=j;   //sur parenthse fermante
	    }
	  }
          courant->set_o(tmp);                              
          courant = tmp;
	}//fin du case 'f'
      }//fin du premier switch
    }//fin du test sur pos    
    pos++; 
  }//fin du while

    //si l'oprateur est nul, prend la premire opration comme opration racine
    if(haut->get_t()=='n') {
      nbop = haut->get_nb_op();
      if(nbop==2) { //deux oprandes sans oprateur
        fprintf(FICHIER_SORTIE,"ERREUR compile - oprateur manquant\n");                  
	return;
        }
      if(nbop==0) {  //fonction vide
        fprintf(FICHIER_SORTIE,"ERREUR compile - opration vide retourne\n");
        return;
	}
      if(haut->get_op(0)!=NULL) {
        courant = haut->get_op(0);
        haut->reset_op(0,NULL); //on met ce pointeurs  zro, sinon le destructeur dtruit tout le reste de l'quation
	delete haut;
        haut = courant; haut->set_pere(NULL);
        }
       //si aucune des conditions ci-dessus ne s'applique c'est que l'opration est une constante !
       //dans ce cas on met le nombre d'oprandes  1
       else haut->set_n(1);
      }
      
  //on copie les membres de haut
  this->cp(haut);  
  }
  
//

