/***************************************************************************
                          qmodel.cpp  -  description
                             -------------------
    begin                : mar avr 6 2004
    copyright            : (C) 2004 by Ferdy Jean-Baptiste
    email                : ferdy@univ-montp2.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "qmodel.h"

#define ECHELLE 10000

extern FILE*  FICHIER_SORTIE;
extern double MAX_FLOAT;
extern int    MAX_T;
extern int    MAX_PERIODE;

qmodele::qmodele(char* texte) : modele(texte)
  {
  pict = new QPixmap(500,500,-1);  //,QPixmap::NoOptim
  initialise_pict();

  coord_standard = new coordonnees(pict);
  coord_phase    = new coordonnees(pict);
  coord_periode  = new coordonnees(pict);
  coord_finetti  = new coordonnees(pict);
  coord_finetti->set_marge(0.05,0.05,0.05,0.05);
  };

qmodele::qmodele(FILE* fichier) : modele(fichier)
  {
     
  pict = new QPixmap(500,500,-1);
  initialise_pict();
  
  coord_standard = new coordonnees(pict);
  coord_phase    = new coordonnees(pict);
  coord_periode  = new coordonnees(pict);
  coord_finetti  = new coordonnees(pict);
  coord_finetti->set_marge(0.1,0.1,0.1,0.1);
  };

qmodele::~qmodele()
  {
  fprintf(FICHIER_SORTIE,"KSimPop -- destruction du modle courant\n");
  delete pict;

  delete coord_standard;
  delete coord_phase;
  delete coord_periode;
  delete coord_finetti;
  };

int qmodele::set_q_var(QTable* tvar)
  {
  int n = get_var_nb();
  char** tab = (char**) malloc(n*sizeof(char*));
  
  
  for(int i=0;i<n;i++) {    
    int j = (tvar->text(i,0)).length();
    tab[i]= (char*) malloc((j+1)*sizeof(char));
    strcpy(tab[i],(tvar->text(i,0)).ascii());
    }
  
  int test = set_var_ini(tab);
      
  if(test!=1) {
    test=-1;
    QMessageBox::information(NULL,"KSimPop","Valeur initiale de variable incorrecte !\nConsultez le fichier log.\n");
    }
  else test=1;
  for(int i=0;i<n;i++) free(tab[i]);
  free(tab);
  return(test);   
  };

int qmodele::set_q_par(QTable* tpar)
  {
  for(int i=0;i<get_par_nb();i++)
    {
    bool ok;
    QString buf = tpar->text(i,0);
    double x = (buf).toDouble(&ok);
    if(ok) parametre(i,x);
    else
      {
      QMessageBox::information( NULL, "KSimPop",
                                "Valeur incorrect pour "+tpar->verticalHeader()->label(i)+".\n");
      return(-1);
      }
    }
  return(1);
  };

int qmodele::set_q_var_par(QTable* tvar,QTable* tpar)
  {
  if(set_q_var(tvar)==-1) return(-1);
  if(set_q_par(tpar)==-1) return(-1);
  return(1);
  };

void qmodele::get_q_var(QTable* tvar)
  {
  tvar->setNumRows(get_var_nb());
  tvar->setColumnStretchable(0,true);
  
  char** tab;
  get_var_ini(&tab);
  
  QString buf;
  for(int i=0;i<get_var_nb();i++)
    {
    buf = get_var_nom(i);
    tvar->verticalHeader()->setLabel(i,buf);    
    tvar->setText(i,0,tab[i]);
    free(tab[i]);
    }
  free(tab);
  };
void qmodele::get_q_par(QTable* tpar)
  {
  tpar->setNumRows(get_par_nb());
  tpar->setColumnStretchable(0,true);    
  QString buf;
  for(int i=0;i<get_par_nb();i++)
    {
    buf = get_par_nom(i);
    tpar->verticalHeader()->setLabel(i,buf);
    buf.setNum(get_par(i),'g',6);
    tpar->setText(i,0,buf);
    }
  };
void qmodele::get_q_var_par(QTable* tvar,QTable* tpar)
  {
  get_q_var(tvar);
  get_q_par(tpar);
  };

void qmodele::ajout_trajectoire(QTable* tvar,QTable* tpar,double tmax)
  {
  if(!valide()) return;

  if(set_q_var_par(tvar,tpar)==-1) return;
  modele::ajout_trajectoire(tmax);
  };

void qmodele::rafraichir_trajectoire(QTable* tpar,double tmax)
  {
  if(!valide()) return;

  if(set_q_par(tpar)==-1) return;
  modele::rafraichir_trajectoire(tmax);
  };

void qmodele::initialise_pict()
  {
  QColor c;
  c.setRgb(255,255,255);
  pict->fill(c);
  }

extern coordonnees* COORD_COURANT;

//dessine trajectoire standard
//--------------------------------------------------------------
void qmodele::dessine_trajectoire_standard()
  {
  initialise_pict();
  if(!valide()) return;
    
  QPainter*  p = new QPainter();
  p->begin(pict,true);
  
  COORD_COURANT = coord_standard;
  COORD_COURANT->calcul_tmax_xymax((modele*) this);
  COORD_COURANT->calcul_echelle();
      
  COORD_COURANT->dessine_axe(p,(char*)"temps",(char*)"");
  
  //affichage des trajectoires    
  for(int i=0;i<get_tr_nb();i++) {
    int      pt_nb = get_trajectoire(i)->get_pt_nb();
    double** v = get_trajectoire(i)->get_v();           
    COORD_COURANT->trace_courbe(p,get_var_nb(),pt_nb,v,get_trajectoire(i)->style());    
    }
  
   
  //affichage des quilibres 
  double* x = (double*) malloc(get_var_nb()*sizeof(double));
  for(int i=0;i<get_eq_nb();i++)
    {
    if(get_eq(i)->fonction::eval(NULL,x)==1) {
    int j;
    if(get_jacob()!=NULL) j=stable(x);
    else                  j=0;
    COORD_COURANT->trace_curseur_equilibre(p,x,get_var_nb(),j);
    }
    }
  free(x);
  
  COORD_COURANT->dessine_legende(p,(modele*) this);   
   
 p->end();                        // painting done
 delete p;
 }

//dessine plan de phase
//--------------------------------------------------------------
void qmodele::dessine_trajectoire_phase()
  {
  initialise_pict();
  if(!valide() || get_var_nb()!=2) return;
  
  QPainter*  p = new QPainter();
  p->begin(pict,true);
  
  initialise_pict();
  COORD_COURANT = coord_phase;
  COORD_COURANT->calcul_xymax((modele*) this);
  COORD_COURANT->calcul_echelle();
  
  COORD_COURANT->dessine_axe(p,get_var_nom(0),get_var_nom(1));  
    
  //affichage des isoclines
  if(get_iso_nb()>0)
    {
    double** range=(double**) malloc(2*sizeof(double*));
    for(int i=0;i<2;i++)
        range[i]=(double*) malloc(3*sizeof(double));
	
    	range[0][0] = COORD_COURANT->get_xmin(); 
	range[0][1] = COORD_COURANT->get_xmax(); 
	range[0][2] = (range[0][1]-range[0][0])/1000.0;
    	
	range[1][0] = COORD_COURANT->get_ymin();
	range[1][1] = COORD_COURANT->get_ymax();
	range[1][2] = (range[1][1]-range[1][0])/1000.0;

    for(int i=0;i<get_iso_nb();i++)
      {
      isocline* iso = get_iso(i);
      int j,l;
      l = iso->get_var_const();  //variable dont la drive s'annule
      j = iso->get_var_dep();    //variable dtermine
      couleur(p,l,2);            //couleur de l'isocline
            
      j = 1-j; //si j est la variable dtermine, 1-j est la variable en fonction de laquelle on calcule
      double** v = iso->get_y()->graphe(j,range[j]); 
      int nb_pt = 1 + (int)((range[j][1]-range[j][0])/range[j][2]);
      COORD_COURANT->trace_courbe(p,j,1-j,nb_pt,v);
            
      for(int k=0;k<nb_pt;k++) free(v[k]);
      free(v);
      }
    for(int i=0;i<2;i++) free(range[i]);
    free(range);
    }
  
  //affichage des trajectoires
  QColor c; 
  c.setRgb(100,100,100); p->setPen(c);  //trajectoires traces en gris
  QPen* lpen = new QPen();
  lpen->setColor(c);
  for(int i=0;i<get_tr_nb();i++) {
    lpen->setWidth(get_trajectoire(i)->style());
    p->setPen(*lpen);    
    COORD_COURANT->trace_courbe_fleche(p,1,2,get_trajectoire(i)->get_pt_nb(),get_trajectoire(i)->get_v());
    lpen->setWidth(0);
    p->setPen(*lpen);
    }
  delete(lpen);

  //affichage des quilibres 
  double* x = (double*) malloc(2*sizeof(double));
  for(int i=0;i<get_eq_nb();i++)
    {
    if(get_eq(i)->fonction::eval(NULL,x)==1) {

    if(x[0]>=0.0 && x[1]>=0.0) {
    	//couleur du point
	int j;
	if(get_jacob()!=NULL) j = stable(x);
	else                  j=0;
	COORD_COURANT->trace_point_equilibre(p,x,get_var_nb(),j);
        }
      }
    }
  free(x);
  
  COORD_COURANT->dessine_legende(p,(modele*) this);
  
  p->end();
  delete p;
  }

//dessine diagramme de de Finetti
//------------------------------------------------------------  
void qmodele::dessine_trajectoire_finetti()
  {
  initialise_pict();
  if(!valide() || get_var_nb()!=3) return;  

  QPainter*  p = new QPainter();;
  p->begin(pict,true);

  QColor c;
  c.setRgb(0,0,0);    QPen* pen_defaut = new QPen (c,2);
    
  COORD_COURANT = coord_finetti;
  COORD_COURANT->calcul_echelle();  
  COORD_COURANT->dessine_triangle(p,get_var_nom(0),get_var_nom(1),get_var_nom(2));
  
  //affichage des trajectoires  
  QPen* lpen   = new QPen ();
  c.setRgb(100,100,100); p->setPen(c);  //trajectoires traces en gris
  lpen->setColor(c);
  for(int i=0;i<get_tr_nb();i++)
    {
    int      pt_nb = get_trajectoire(i)->get_pt_nb();
    double** v = get_trajectoire(i)->get_v();
    
    lpen->setWidth(get_trajectoire(i)->style());
    p->setPen(*lpen);
    
    for(int j=1;j<pt_nb;j++)          
      COORD_COURANT->trace_ligne(p,v[j-1][1],v[j-1][2],v[j-1][3],v[j][1],v[j][2],v[j][3]);     // trace la ligne  
    
    lpen->setWidth(0);
    p->setPen(*lpen);    
    }
  delete(lpen);
  
  //affichage des isoclines
  if(get_iso_nb()>0)
    {
    
    int    pt_nb = 1000;
    double ymin  = COORD_COURANT->get_ymin(); 
    double ymax  = COORD_COURANT->get_ymax();    
    double ystep = (ymax-ymin)/(double)pt_nb;
    
    double* x0 = (double*) malloc(3*sizeof(double));
    double* x1 = (double*) malloc(3*sizeof(double));
    
    for(int i=0;i<get_iso_nb();i++)
      {
      isocline* iso = get_iso(i);
      switch(iso->get_y()->get_var_dep()) {
        case 0 : { //une des trois variables est constante
		int j,l;
      		l = iso->get_var_const();  //variable dont la drive s'annule
      		couleur(p,l,2);            //couleur de l'isocline
		j = iso->get_var_dep();    //variable dtermine
                  
      		double x = ymin;
      		double y;
		if(iso->get_y()->fonction::eval(&y)==-1) break;      
            
      		for(x=ymin;x<ymax;x+=ystep) {
			double z = ymax-x-y;
	  		switch(j) {
				case 0: {COORD_COURANT->trace_ligne(p,y,x,z,y,x+ystep,z-ystep); break;}
				case 1: {COORD_COURANT->trace_ligne(p,x,y,z,x+ystep,y,z-ystep); break;}
				case 2: {COORD_COURANT->trace_ligne(p,z,x,y,z-ystep,x+ystep,y); break;}
				}
			}
		break;
		}
      	case 1 : { //une variable dpend d'une autre variable
      	int j,l;
      	l = iso->get_var_const();  //variable dont la drive s'annule
      	j = iso->get_var_dep();    //variable dtermine
      	couleur(p,l,2);            //couleur de l'isocline
                                          
      	for(int k=0;k<3;k++) {x0[k]=ymin; x1[k]=ymin;}      
      	iso->get_y()->set_var(x0);
	if(iso->get_y()->fonction::eval(&(x0[j]))==-1) break;
	          
      	double x = ymin;
      
      	int k;      
      	for(k=0;iso->get_y()->is_var_dep(k)==0 && k<3;k++); //variable dpendante
            
      	for(x=ymin+ystep;x<=ymax;x+=ystep) {
          		x1[k] = x;
	  		iso->get_y()->set_var(x1);
	        	if(iso->get_y()->fonction::eval(&(x1[j]))==1) {
	  			if(j!=0&&k!=0) x1[0] = ymax-x1[1]-x1[2]; 
	  			if(j!=1&&k!=1) x1[1] = ymax-x1[0]-x1[2];
	  			if(j!=2&&k!=2) x1[2] = ymax-x1[0]-x1[1];
	  
	  			COORD_COURANT->trace_ligne(p,x0[0],x0[1],x0[2],x1[0],x1[1],x1[2]);

	  			x0[0]=x1[0]; x0[1]=x1[1]; x0[2]=x1[2];
				}
			}
		break;
		}
	default : {
	//isocline ignor car impossible  afficher
	fprintf(FICHIER_SORTIE,"AVERTISSEMENT qmodele::dessine_trajectoire_finetti - isocline %d fonction de plus d'une variable impossible  afficher.\n",i);
		}
	}
	}       
	free(x0); free(x1);
}
    
        
  //affichage des quilibres
  c.setRgb(0,0,0);
  p->setPen(c);
  double* x = (double*) malloc(3*sizeof(double));  
  for(int i=0;i<get_eq_nb();i++) {
    if(get_eq(i)->fonction::eval(NULL,x)==1) {
        int j;
	if(get_jacob()!=NULL) j = stable(x);
	else                  j=0;
        COORD_COURANT->trace_point_equilibre(p,x,get_var_nb(),j);
    	}
    }
  free(x);
  
 

  p->end();
  delete p;
  delete pen_defaut;
  }

//dessine graphique nt en fonction nt+T
//----------------------------------------------------------------  
void qmodele::dessine_trajectoire_periode(int periode,int delai)
  {
  initialise_pict();
  if(!valide() || get_var_nb()!=1 || periode<1) return;  
  
  QPainter*  p = new QPainter();
  p->begin(pict,true);

  QColor c; c.setRgb(80,80,80); 
  QPen* pen_traj = new QPen (c,1);
      
  COORD_COURANT = coord_periode;
  COORD_COURANT->calcul_xmax((modele*) this);
  COORD_COURANT->calcul_echelle();
  
  QString buf = get_var_nom(0);
  COORD_COURANT->dessine_axe(p,(char*)(buf+"(t)").ascii(),(char*)(buf+"(t+T)").ascii());
  COORD_COURANT->trace_ligne(p,coord_periode->get_xmin(),coord_periode->get_xmin(),coord_periode->get_xmax(),coord_periode->get_ymax());
     
  //affichage des trajectoires
  p->setPen(*pen_traj);
  c.setRgb(80,80,80); p->setBrush(c);
  
  for(int i=0;i<get_tr_nb();i++)
    {
    int      pt_nb = get_trajectoire(i)->get_pt_nb();
    double** v = get_trajectoire(i)->get_v();

    for(int j=delai+periode;j<pt_nb;j++)
      {
      if(COORD_COURANT->point_valide(v[j-periode][1],v[j][1]))
      	COORD_COURANT->trace_point(p,v[j-periode][1],v[j][1]);      
      }
    }

  p->end();
  delete p;
  }

