/***************************************************************************
 *   Copyright (C) 2005 by Jean-Baptiste Ferdy                             *
 *   ferdy@univ-montp2.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "sfonction.h"

extern FILE*  FICHIER_SORTIE;
extern double ODE_PRECISION;
extern double MAX_FLOAT;        //valeur max des variables permise (lue dans un fichier)
extern int    MAX_TEMPS_CALCUL;
extern int    PROBLEME_CALCUL;

//---------------------------------------------------------------
//constructeur et destructeur
//---------------------------------------------------------------
sfonction::sfonction(char* sv,char* sp,char** sf) : fonction(sv,sp,sf) { 
 var_stochastique=0;
 par_stochastique=0;
 var_dist  = NULL;
 par_dist  = NULL;
 var_tmp   = NULL;
 par_tmp_1 = NULL;
 par_tmp_2 = NULL;
 x_tmp     = NULL;
}

sfonction::sfonction(int l_nb,int l_var_nb,int l_par_nb,char** var_nom,char** par_nom,double* var,double* par,char** s) : fonction(l_nb,l_var_nb,l_par_nb,var_nom,par_nom,var,par,s){
 
 var_stochastique=0;
 par_stochastique=0;
 var_dist = NULL;
 par_dist = NULL;
 var_tmp   = (double*) malloc(this->get_var_nb()*sizeof(double));
 par_tmp_1 = (double*) malloc(this->get_par_nb()*sizeof(double)); 
 par_tmp_2 = (double*) malloc(this->get_par_nb()*sizeof(double));
 x_tmp     = (double*) malloc(this->get_nb()*sizeof(double));
}

sfonction::~sfonction() {
	if(var_dist!=NULL) {
		for(int i=0;i<get_var_nb();i++) {
			if(var_dist[i]!=NULL) delete var_dist[i];
			}
		free(var_dist);
		var_dist = NULL;
		}
	var_stochastique=0;
	if(par_dist!=NULL) {
		for(int i=0;i<get_par_nb();i++) {
			if(par_dist[i]!=NULL) delete par_dist[i];
			}
		free(par_dist);
		par_dist = NULL;
		}
	par_stochastique=0;
	if(var_tmp!=NULL)   {free(var_tmp);   var_tmp=NULL;}
	if(par_tmp_1!=NULL) {free(par_tmp_1); par_tmp_1=NULL;}	
	if(par_tmp_2!=NULL) {free(par_tmp_2); par_tmp_2=NULL;}
	if(x_tmp!=NULL)     {free(x_tmp);     x_tmp=NULL;}
  }
	
void sfonction::set_var_dist(distribution** d) {
var_dist = d;
if(d == NULL) var_stochastique = 0;
}
	
void sfonction::set_par_dist(distribution** d) {
par_dist = d;
if(d == NULL) par_stochastique = 0;
}

void sfonction::set_var_stochastique(int i)
{
if(i==1) var_stochastique=1;
else     var_stochastique=0;
};

void sfonction::set_par_stochastique(int i){
if(i==1) par_stochastique=1;
else     par_stochastique=0;
};
  
//affiche une fonction
void sfonction::ecrire() {  
  if(est_matriciel()==0) for(int i=0;i<get_var_nb();i++) {get_f(i)->ecrire(); fprintf(FICHIER_SORTIE,"\n");}
  else {
    for(int i=0;i<get_nb();i++) {
	for(int j=0;j<get_var_nb();j++) {
          operation* ftmp = get_f(i);
	  if(ftmp!=NULL) ftmp->ecrire();
	  if(j<get_var_nb()-1) fprintf(FICHIER_SORTIE,",\t");
	  i++;
          }
	  i--;
	fprintf(FICHIER_SORTIE,"\n");
	}
    }
  
  if(var_stochastique==1) {
  	for(int i=0;i<get_var_nb();i++) {
		if(var_dist[i]!=NULL) var_dist[i]->ecrire(get_var_nom(i));
		}
  	}
  if(par_stochastique==1) {
  	for(int i=0;i<get_par_nb();i++) {
		if(par_dist[i]!=NULL) par_dist[i]->ecrire(get_par_nom(i));
		}
  	}
  };  

void sfonction::ecrire(char** nom) {  
  if(est_matriciel()==0) for(int i=0;i<get_var_nb();i++) {
    fprintf(FICHIER_SORTIE,"%s = ",nom[i]);
    get_f(i)->ecrire(); 
    fprintf(FICHIER_SORTIE,"\n");
    }
  else {
    for(int i=0;i<get_nb();i++) {
	for(int j=0;j<get_var_nb();j++) {          
	  operation* ftmp = get_f(i);
	  if(ftmp!=NULL) ftmp->ecrire();
	  if(j<get_var_nb()-1) fprintf(FICHIER_SORTIE,",\t");
	  i++;
          }
	  i--;
	fprintf(FICHIER_SORTIE,"\n");
	}
    }
  
  if(var_stochastique==1) {
  	for(int i=0;i<get_var_nb();i++) {
		if(var_dist[i]!=NULL) var_dist[i]->ecrire(get_var_nom(i));
		}
  	}
  if(par_stochastique==1) {
  	for(int i=0;i<get_par_nb();i++) {
		if(par_dist[i]!=NULL) par_dist[i]->ecrire(get_par_nom(i));
		}
  	}
  };

int sfonction::eval(double* x,double* y) {

    set_var(x);

    //tirage al�toire des valeurs de paramètre
    if(par_stochastique==1) {
    	for (int i=0;i<get_par_nb();i++) {
		par_tmp_1[i] = get_par(i);
		par_tmp_2[i] = par_tmp_1[i];
		if(par_dist[i]!=NULL) {
    			if(par_dist[i]->eval(&(par_tmp_1[i]))==-1) {PROBLEME_CALCUL=-1; return(-1);};
    			//on met à jour les valeurs de paramètre au fur et à mesure
    			//de cette façon un paramètre sera calculée avec les valeurs de paramètre déjà mises à jour
    			set_par(i,par_tmp_1[i]);
    			}
		}
	//set_par(par_tmp_1);
       }

    int test = fonction::eval(x,y);
    if(test!=1) return(test);
 
    //tirage al�toire des valeurs des variables
    if(var_stochastique==1) {
	for (int i=0;i<get_var_nb();i++) {
	  if(var_dist[i]!=NULL) {
	    if(var_dist[i]->eval(&(y[i]))==-1) {PROBLEME_CALCUL=-1; return(-1);};
            }	  
          if(y[i]>MAX_FLOAT || y[i]<-MAX_FLOAT) {PROBLEME_CALCUL=1; return(-1);}
          }
      }

    if(par_stochastique==1) set_par(par_tmp_2); //on affecte les ancienes valeurs aux paramètres

    return(1);
  }

int sfonction::eval(double* x) {
  double* y = (double*) malloc(get_nb()*sizeof(double));
  int test = eval(x,y);
  for(int i=0;i<get_nb();i++) x[i]=y[i];
  free(y);
  return(test);
  }

int sfonction::meval(double* x,double* y) {
    set_var(x);

    //tirage al�toire des valeurs de param�re
    if(par_stochastique==1) {        
    	for (int i=0;i<get_par_nb();i++) {
		par_tmp_1[i] = get_par(i);
		par_tmp_2[i] = par_tmp_1[i];
		if(par_dist[i]!=NULL) {
    			if(par_dist[i]->eval(&(par_tmp_1[i]))==-1) {PROBLEME_CALCUL=-1; return(-1);};
    			//on met à jour les valeurs de paramètre au fur et à mesure
    			//de cette façon un paramètre sera calculée avec les valeurs de paramètre déjà mises à jour
    			set_par(i,par_tmp_1[i]);
    			}
		}
	set_par(par_tmp_1);
       }

    int test = fonction::meval(x,y);
    if(test!=1) return(test);        

    //tirage al�toire des valeurs des variables
    if(var_stochastique==1) {      
	for (int i=0;i<get_var_nb();i++) {
	  if(var_dist[i]!=NULL) {
	    if(var_dist[i]->eval(&(y[i]))==-1) {PROBLEME_CALCUL=-1; return(-1);};
            }
          if(x[i]>MAX_FLOAT || x[i]<-MAX_FLOAT) {PROBLEME_CALCUL=1; return(0);}
          }
      }

    if(par_stochastique==1) set_par(par_tmp_2); //on affecte les ancienes valeurs aux paramètres

    return(1);
  }

int sfonction::meval(double* x) {
  double* y = (double*) malloc(get_var_nb()*sizeof(double));
  int test = meval(x,y);
  for(int i=0;i<get_var_nb();i++) x[i]=y[i];
  free(y);
  return(test);  
  }
  
//
