/***************************************************************************
 *   Copyright (C) 2005 by Jean-Baptiste Ferdy                             *
 *   ferdy@univ-montp2.fr                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

 
 #include "specification_fonction.h"
 #include <time.h>
 
 extern FILE*  FICHIER_SORTIE;
 extern char   MSG_PROBLEME_CALCUL[];
 
 #define  NB_FONC 70
 
 char           SPEC_FONC_NOM[NB_FONC][25];
 char           SPEC_FONC_TYPE[NB_FONC];
 unsigned int   SPEC_FONC_SYMB[NB_FONC];
 unsigned int   SPEC_FONC_NMIN[NB_FONC];
 unsigned int   SPEC_FONC_NMAX[NB_FONC]; 
 func_pt        SPEC_FONC_F[NB_FONC];
 
 gsl_rng        *RNG;  //gnrateur de nombre alatoire dfini comme une variable globale
		       //il n'est donc alou qu'une seule fois, dans la routine qui charge les spcifications de fonction
 
 //********************************************************************************************************
 //routines d'valuation
 //********************************************************************************************************
 
 double evaluation_addition(int n,double* x,int* test) {
   if(n<=0) {*test = -1; return(0);}
   double z = 0.0;
   for(int i=0;i<n;i++) z += x[i];   
   *test = 1;
   return(z);
   }
   
 double evaluation_soustraction(int n,double* x,int* test){
   if(n==1) {*test=1; return(-x[0]);}
   if(n==2) {*test=1; return(x[0]-x[1]);}
   *test = -1;
   return(0);
   }
   
 double evaluation_multiplication(int n,double* x,int* test){
   if(n<=0) {*test = -1; return(0);}
   double z = 1.0;
   for(int i=0;i<n;i++) z *= x[i];   
   *test = 1;
   return(z);   
   }
   
 double evaluation_division(int n,double* x,int* test){
   if(n!=2) {*test = -1; return(0);}
   if(x[1]==0) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR evaluation_division - division par zro\n");
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     *test = -1;
     return(0);
     }
   *test=1; 
   return(x[0]/x[1]);   
   }
 
 double evaluation_puissance(int n,double* x,int* test){
   if(n!=2) {*test = -1; return(0);}   
   *test=1; 
   return(pow(x[0],x[1]));   
   }
   
 double evaluation_racine(int n,double* x,int* test){
   if(n!=1) {*test = -1; return(0);}   
   if(x[0]<0) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR evaluation_racine - sqrt(x) non dfini pour x<0\n");
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     *test = -1;
     return(0);
     }
   *test=1; 
   return(sqrt(x[0]));   
   }
       
 double evaluation_exp(int n,double* x,int* test) {
   if(n<=0) {*test = -1; return(0);}
   *test = 1;
   return(exp(x[0]));
   }
   
 double evaluation_log(int n,double* x,int* test) {
   if(n<=0) {*test = -1; return(0);}
   if(x[0]<=0) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR evaluation_log - log(x) non dfini pour x<=0\n");
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     *test = -1;
     return(0);
     }
   *test = 1;
   return(log(x[0]));
   }
 
 double evaluation_factoriel(int n,double* x,int* test){
   if(n!=1) {*test = -1; return(0);}   
   if(x[0]==1.0) return(1.0);
   if(x[0]<1) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR evaluation_factoriel - fonction gamma non dfinie pour x<0\n");
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     *test = -1;
     return(0);
     }
   if(x[0]>171.0) { //valeur max donne dans la doc de gsl
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR evaluation_factoriel - dpassement de capacit dans la fonction gamma pour x>=171.0\n");
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     *test = -1;
     return(0);
     }  
   *test=1;    
   return(gsl_sf_gamma(x[0]-1));   
   }
       
 double evaluation_abs(int n,double* x,int* test){
   if(n<=0) {*test = -1; return(0);}   
   *test = 1;
   return(fabs(x[0]));
   }
   
 double evaluation_sin(int n,double* x,int* test){
   if(n<=0) {*test = -1; return(0);}   
   *test = 1;
   return(sin(x[0]));
   }
   
 double evaluation_cos(int n,double* x,int* test){
   if(n<=0) {*test = -1; return(0);}   
   *test = 1;
   return(cos(x[0]));
   }
  
 double evaluation_maximum(int n,double* x,int* test) {
  if(n<=0) {*test = -1; return(0);}
  double z = x[0];
  for(int i=1;i<n;i++) if(x[i]>z) z = x[i];
  return(z);
 }
 
 double evaluation_minimum(int n,double* x,int* test){
  if(n<=0) {*test = -1; return(0);}
  double z = x[0];
  for(int i=1;i<n;i++) if(x[i]<z) z = x[i];
  return(z);
 }
 
 //distributions probabilistes************************************************************************
 //distributios discrtes*****************************************************************************
 //uniforme discrete
 int parametre_uniforme_discrete(int n,double* x) {   
   if(n<0 || n>2) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_uniforme_discrete - impossible de dfinir une uniforme avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   
   if(x[0]<0) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_uniforme_discrete - les paramtres de la distribution uniforme doivent tre positifs !\n");
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);     
     return(-1);
     } 
   if(n==2 && x[1]<0) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_uniforme_discrete - les paramtres de la distribution uniforme doivent tre positifs !\n");
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);     
     return(-1);
     } 
   if(n==2 && x[0]==x[1]) { 
      sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_uniforme_discrete - impossible de dfinir une uniforme entre %d et %d\n",(int)(x[0]),(int)(x[1]));
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);     
     } 
   if(n==2 && x[0]>x[1]) {
     double z = x[0];
     x[0] = x[1];
     x[1] = z;
     fprintf(FICHIER_SORTIE,"AVERTISSEMENT parametre_uniforme_discrete - les bornes de la distribution uniforme ont t permute.\n");
     }
   return(1);
   }   
 double evaluation_fd_uniforme_discrete(int n,double* x,int* test){
   *test = parametre_uniforme_discrete(n-1,&(x[1]));
   if((*test)==-1) return(0.0);
   if(x[0]<0 || (n==3 && x[0]<x[1]) || (n==2 && x[0]>x[1]) || (n==3 && x[0]>x[2])) return(0);
   switch(n) {
     case 1 : {return(0.5);}
     case 2 : {return(1/(x[1]+1));}
     case 3 : {return(1/(x[2]-x[1]+1));}
     }   
   return(0.0);
   }
 double evaluation_fp_uniforme_discrete(int n,double* x,int* test){
   *test = parametre_uniforme_discrete(n-1,&(x[1]));
   if((*test)==-1) return(0.0);
   if(x[0]<0 || (n==3 && x[0]<x[1])) return(0);
   if((n==2 && x[0]>=x[1]) || (n==3 && x[0]>=x[2])) return(1);
   if(n==2) return((x[0]+1)/(double)(n-1));
   switch(n) {
     case 1 : {return(0.5);}
     case 2 : {return((x[0]+1)/(x[1]-x[0]+1));}
     case 3 : {return((x[0]-x[1]+1)/(x[2]-x[1]+1));}
     }   
   return(0.0);
   }
 double evaluation_fr_uniforme_discrete(int n,double* x,int* test){
   *test = parametre_uniforme_discrete(n,x);
   if((*test)==-1) return(0.0);
   int z;
   switch(n) {
    case 0: {z = gsl_rng_uniform_int(RNG,2); break;}
    case 1: {z = gsl_rng_uniform_int(RNG,(int)x[0]+1); break;}
    case 2: {z = (int)x[0]+gsl_rng_uniform_int(RNG,(int)(x[1]-x[0]+1)); break;}
    default : z=0;
    }
   return(z);
   }   
 //bernoulli
 int parametre_bernoulli(int n,double* x) {
   if(n<1) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_bernoulli - impossible de dfinir une Bernoulli avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   if(x[0]<0 || x[0]>1) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_bernoulli - impossible de dfinir une Bernoulli avec p=%g.\n",x[0]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }  
   return(1);
   }
 double evaluation_fd_bernoulli(int n,double* x,int* test){
   *test = parametre_bernoulli(n-1,&(x[1]));
   if((*test)==-1) return(0);
   if(x[0]==0) return(1-x[1]);
   return(x[1]);
   }
 double evaluation_fp_bernoulli(int n,double* x,int* test){
   *test = parametre_bernoulli(n-1,&(x[1]));
   if((*test)==-1) return(0);
   if(x[0]==0) return(1-x[1]);
   return(1.0);
   }
 double evaluation_fr_bernoulli(int n,double* x,int* test){
   *test = parametre_bernoulli(n,x);
   if((*test)==-1) return(0);
   double z = gsl_rng_uniform(RNG);
   if(z<x[0]) return(1);
   return(0);
   }
 //bernouilli multiple
 int parametre_ddiscrete(int n,double *x) {
   if(n<1){
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_ddiscrete - impossible de dfinir une distribution discrte avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }     
   return(1);
   }
 double evaluation_fd_ddiscrete(int n,double* x,int* test){
   *test = parametre_ddiscrete(n-1,&(x[1]));
   if((*test)==-1) return(0.0);      
   if(x[0]<0)   return(0.0);   
   double w = 0.0;   
   for(int i=0;i<n;i++) {w+=x[i+1];}
   if(w<1.0) {
     if(x[0]>n-3) return(0.0);
     if(x[0]==0) return(1.0-w);
     return(x[(int)(x[0])]);
     }
   
   if(x[0]>n-2) return(0.0);
   return(x[(int)(x[0])+1]);   
   }
 double evaluation_fp_ddiscrete(int n,double* x,int* test){
   *test = parametre_ddiscrete(n-1,&(x[1]));
   if((*test)==-1) return(0);   
   if(x[0]<0)   return(0.0);   
   double w = 0.0;
   double z = 0.0;   
   for(int i=0;i<n;i++) {w+=x[i+1]; if(i<=x[0]) z+=x[i+1];}
   if(w<1.0) {     
     if(x[0]>n-3) return(1.0);
     return(z+1.0-w-x[(int)(x[0])+1]);
     }
   if(x[0]>n-2) return(1.0);      
   return(z);
   }
 double evaluation_fr_ddiscrete(int n,double* x,int* test){
   *test = parametre_ddiscrete(n,x);
   if((*test)==-1) return(0);
   double z = gsl_rng_uniform(RNG);
   double sw = 0.0;
   double w  = 0.0;   
   int i;
   for(i=0;i<n;i++) sw += x[i];
   for(i=0;i<n && z>w;i++) w+=x[i]/sw;
   return(i-1);
   } 
 //poisson
 int parametre_poisson(int n,double* x) {
   if(n<1) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_poisson - impossible de dfinir une Poisson avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   if(x[0]<0) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_poisson - Poisson non dfinie pour m=%g\n",x[1]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);     
     return(-1);
     }
   return(1);
   }      
 double evaluation_fd_poisson(int n,double* x,int* test){   
   *test = parametre_poisson(n-1,&(x[1]));
   if((*test)==-1) return(0);
   int k = (int) x[0];
   return(gsl_ran_poisson_pdf(k,x[1]));
   }   
 double evaluation_fp_poisson(int n,double* x,int* test){   
   *test = parametre_poisson(n-1,&(x[1]));
   if((*test)==-1) return(0);      
   int k = (int) x[0];
   double z = 0.0;
   for(int i=0;i<=k;i++) z += gsl_ran_poisson_pdf(i,x[1]);
   return(z);
   }   
double evaluation_fr_poisson(int n,double* x,int* test){  
   *test = parametre_poisson(n,x);
   if((*test)==-1) return(0);         
   return(gsl_ran_poisson(RNG,x[0]));
   }
 //binomiale
 int parametre_binomiale(int n,double* x) {
   if(n<2) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_binomiale - impossible de dfinir une binomiale avec %d argument(s)\n",n);
     return(-1);
     }
   if(x[0]<0 || x[0]>1) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_binomiale - binomiale non dfinie pour p=%g\n",x[0]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);     
     return(-1);
     }
   if(x[1]<0) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_binomiale - binomiale non dfinie pour N=%d\n",(int) x[1]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);     
     return(-1);
     }      
   return(1);
   }   
 double evaluation_fd_binomiale(int n,double* x,int* test){
   *test = parametre_binomiale(n-1,&(x[1]));
   if((*test)==-1) return(0);
   int k     = (int) (x[0]);
   double p  = x[1];
   int    nb = (int) (x[2]);                
   if(k>nb || k<0) return(0.0);
   if(p<=0.0) {
     if(k==0) return(1.0); else return(0.0);
     }
   if(p>=1.0) {
     if(k==nb) return(1.0); else return(0.0);
     }     
   return(gsl_ran_binomial_pdf(k,p,nb));
   }   
 double evaluation_fp_binomiale(int n,double* x,int* test){
   *test = parametre_binomiale(n-1,&(x[1]));
   if((*test)==-1) return(0);   
   int k     = (int) (x[0]);
   double p  = x[1];
   int    nb = (int) (x[2]);       
   if(k>nb || k<0) return(0.0);
   if(p<=0.0) {
     if(k>=0) return(1.0); return(0.0);
     }
   if(p>=1.0) {
     if(k>=nb) return(1.0); return(0.0);
     }
   double z = 0;
   for(int i=0;i<=k;i++) z += gsl_ran_binomial_pdf(i,p,nb);
   return(z);
   }   
 double evaluation_fr_binomiale(int n,double* x,int* test){
   *test = parametre_binomiale(n,x);
   if((*test)==-1) return(0);   
   double p  = x[0];
   int    nb = (int) (x[1]);
   return(gsl_ran_binomial(RNG,p,nb));
   }
 //hypergeometrique
 int parametre_hypergeometrique(int n,double* x) {
   if(n<3) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_hypergeometrique - impossible de dfinir une hypergeomtrique avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);     
     }
   if(x[0]<0 || x[1]<0 || x[2]<0 || x[2]>(x[0]+x[1])) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_hypergeometrique - impossible de dfinir une hypergeomtrique avec n1=%d, n2=%d et n=%d\n",(int)x[0],(int)x[1],(int)x[2]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   return(1);
   }
 double evaluation_fd_hypergeometrique(int n,double* x,int* test){
   *test = parametre_hypergeometrique(n-1,&(x[1]));
   if((*test)==-1) return(0);
   return(gsl_ran_hypergeometric_pdf((int)x[0],(int)x[1],(int)x[2],(int)x[3]));
   }
 double evaluation_fp_hypergeometrique(int n,double* x,int* test){
   *test = parametre_hypergeometrique(n-1,&(x[1]));
   if((*test)==-1) return(0);
   double z=0.0;
   for(int i=0;i<=x[0];i++) z+=gsl_ran_hypergeometric_pdf(i,(int)x[1],(int)x[2],(int)x[3]);
   return(z);
   }
 double evaluation_fr_hypergeometrique(int n,double* x,int* test){
   *test = parametre_hypergeometrique(n,x);
   if((*test)==-1) return(0);
   return(gsl_ran_hypergeometric(RNG,(int)x[0],(int)x[1],(int)x[2]));   
   }
 //binomiale negative
 int parametre_binomiale_negative(int n,double* x) {
   if(n<2) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR binomiale_negative - impossible de dfinir une binomiale ngative avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);     
     }
   if(x[0]<0 || x[0]>1)  {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR binomiale_negative - impossible de dfinir une binomiale ngative avec p=%g.\n",x[0]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);     
     } 
   if(x[1]<0)  {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR binomiale_negative - impossible de dfinir une binomiale ngative avec n=%g.\n",x[1]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);     
     }
   return(1);
   }
 double evaluation_fd_binomiale_negative(int n,double* x,int* test){
   *test = parametre_binomiale_negative(n-1,&(x[1]));
   if((*test)==-1) return(0.0);
   return(gsl_ran_negative_binomial_pdf((int)x[0],x[1],x[2]));
   }
 double evaluation_fp_binomiale_negative(int n,double* x,int* test){
   *test = parametre_binomiale_negative(n-1,&(x[1]));
   if((*test)==-1) return(0.0);
   double z = 0;
   for(int i=0;i<=x[0];i++) z += gsl_ran_negative_binomial_pdf(i,x[1],x[2]);
   return(z);
   }
 double evaluation_fr_binomiale_negative(int n,double* x,int* test){
   *test = parametre_binomiale_negative(n,x);
   if((*test)==-1) return(0.0);
   return(gsl_ran_negative_binomial(RNG,x[0],x[1]));
   }
 //geomtrique
 int parametre_geometrique(int n,double* x) {
   if(n<1) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_geometrique - impossible de dfinir une gomtrique avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);     
     }
   if(x[0]<0 || x[0]>1)  {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_geometrique - impossible de dfinir une gomtrique avec p=%g.\n",x[0]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);     
     } 
   return(1);
   }
 double evaluation_fd_geometrique(int n,double* x,int* test){
   *test = parametre_geometrique(n-1,&(x[1]));
   if((*test)==-1) return(0.0);
   return(gsl_ran_geometric_pdf((unsigned int)(x[0]),x[1]));
   }
 double evaluation_fp_geometrique(int n,double* x,int* test){
   *test = parametre_geometrique(n-1,&(x[1]));
   if((*test)==-1) return(0.0);
   double z = 0;
   for(int i=0;i<=x[0];i++) z += gsl_ran_geometric_pdf(i,x[1]);
   return(z);
   }
 double evaluation_fr_geometrique(int n,double* x,int* test){
   *test = parametre_geometrique(n,x);
   if((*test)==-1) return(0.0);
   return(gsl_ran_geometric(RNG,x[0])-1);
   } 
 //distributions probabilistes continues
 //uniforme
 int parametre_uniforme_continue(int n,double* x) {   
   if(n<0 || n>2) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_uniforme_continue - impossible de dfinir une uniforme avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   if(n==2 && x[0]==x[1]) { 
      sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_uniforme_continue - impossible de dfinir une uniforme entre %g et %g\n",x[0],x[1]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);     
     }
   if(n==2 && x[0]>x[1]) {
     double z = x[0];
     x[0] = x[1];
     x[1] = z;
     fprintf(FICHIER_SORTIE,"AVERTISSEMENT parametre_uniforme_continue - les bornes de la distribution uniforme ont t permute.\n");
     }
   return(1);
   } 
 double evaluation_fd_uniforme_continue(int n,double* x,int* test){
   *test = parametre_uniforme_continue(n-1,&(x[1]));
   if((*test)==-1) return(0.0);   
   switch(n) {
     case 1 : {return(gsl_ran_flat_pdf(x[0],0.0,1.0));}
     case 2 : {return(gsl_ran_flat_pdf(x[0],0.0,x[1]));}
     case 3 : {return(gsl_ran_flat_pdf(x[0],x[1],x[2]));}
     }      
   return(0.0);
   }
 double evaluation_fp_uniforme_continue(int n,double* x,int* test){
   *test = parametre_uniforme_continue(n-1,&(x[1]));
   if((*test)==-1) return(0.0);
   switch(n) {
     case 1 : {return(gsl_cdf_flat_P(x[0],0.0,1.0));}
     case 2 : {return(gsl_cdf_flat_P(x[0],0.0,x[1]));}
     case 3 : {return(gsl_cdf_flat_P(x[0],x[1],x[2]));}
     }      
   return(0.0);
   }
 double evaluation_fr_uniforme_continue(int n,double* x,int* test){
   *test = parametre_uniforme_continue(n,x);
   if((*test)==-1) return(0.0);   
   switch(n) {
     case 0: {return(gsl_ran_flat(RNG,0.0,1.0));}
     case 1: {return(gsl_ran_flat(RNG,0.0,x[0]));}
     case 2: {return(gsl_ran_flat(RNG,x[0],x[1]));}
     }
   return(0.0);   
   } 
 //normale
 int parametre_normale(int n,double* x) {
   if(n<2) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_normale - impossible de dfinir une normale avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   if(x[1]<=0) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_normale - impossible de dfinir une normale avec sqrt(var)=%g\n",x[1]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   return(1);
   }
 double evaluation_fd_normale(int n,double* x,int* test){
   *test = parametre_normale(n-1,&(x[1]));
   if((*test)==-1) return(0);
   return(gsl_ran_gaussian_pdf(x[0]-x[1],x[2]));
   }
 double evaluation_fp_normale(int n,double* x,int* test){
   *test = parametre_normale(n-1,&(x[1]));
   if((*test)==-1) return(0);
   return(gsl_cdf_gaussian_P(x[0]-x[1],x[2]));
   }
 double evaluation_fr_normale(int n,double* x,int* test){
   *test = parametre_normale(n,x);
   if((*test)==-1) return(0);
   return(x[0]+gsl_ran_gaussian(RNG,x[1]));
   }   
 //log normale
 int parametre_lognormale(int n,double* x) {
   if(n<2) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_lognormale - impossible de dfinir une log-normale avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   return(1);
   }
 double evaluation_fd_lognormale(int n,double* x,int* test){
   *test = parametre_lognormale(n-1,&(x[1]));
   if((*test)== -1) return(0.0);   
   return(gsl_ran_lognormal_pdf(x[0],x[1],x[2]));
   }
 double evaluation_fp_lognormale(int n,double* x,int* test){
   *test = parametre_lognormale(n-1,&(x[1]));
   if((*test)== -1) return(0.0);
   return(gsl_cdf_lognormal_P(x[0],x[1],x[2]));
   }
 double evaluation_fr_lognormale(int n,double* x,int* test){
   *test = parametre_lognormale(n,x);
   if((*test)== -1) return(0.0);
   return(gsl_ran_lognormal(RNG,x[0],x[1]));
   }
 //exponentielle
 int parametre_exponentielle(int n,double* x) {
   if(n<1) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_exponentielle - impossible de dfinir une exponentielle avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   if(x[0]<=0) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_normale - impossible de dfinir une exponentielle avec pour moyenne %g.\n",x[1]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   return(1);
   }
 double evaluation_fd_exponentielle(int n,double* x,int* test){
   *test = parametre_exponentielle(n-1,&(x[1]));
   if((*test)== -1) return(0.0);
   return(gsl_ran_exponential_pdf(x[0],x[1]));
   }        
 double evaluation_fp_exponentielle(int n,double* x,int* test){
   *test = parametre_exponentielle(n-1,&(x[1]));
   if((*test)== -1) return(0.0);
   return(gsl_cdf_exponential_P(x[0],x[1]));
   }        
 double evaluation_fr_exponentielle(int n,double* x,int* test){
   *test = parametre_exponentielle(n,x);
   if((*test)== -1) return(0.0);
   return(gsl_ran_exponential(RNG,x[0]));
   } 
          
 //gamma
 int parametre_gamma(int n,double* x) {
   if(n<2) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_gamma - impossible de dfinir une gamma avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   return(1);
   }
 double evaluation_fd_gamma(int n,double* x,int* test){
   *test = parametre_gamma(n-1,&(x[1]));
   if((*test)== -1) return(0.0);
   return(gsl_ran_gamma_pdf(x[0],x[1],x[2]));
   }
 double evaluation_fp_gamma(int n,double* x,int* test){
   *test = parametre_gamma(n-1,&(x[1]));
   if((*test)== -1) return(0.0);
   return(gsl_cdf_gamma_P(x[0],x[1],x[2]));
   }
 double evaluation_fr_gamma(int n,double* x,int* test){
   *test = parametre_gamma(n,x);
   if((*test)== -1) return(0.0);
   return(gsl_ran_gamma(RNG,x[0],x[1]));
   }        
 //beta
 int parametre_beta(int n,double* x) {
   if(n<2) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_beta - impossible de dfinir une beta avec %d argument(s)\n",n);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   if(x[0]<0 || x[1]<0) {
     sprintf(MSG_PROBLEME_CALCUL,"ERREUR parametre_beta - impossible de dfinir une beta avec a=%g et b=%g\n",x[0],x[1]);
     fprintf(FICHIER_SORTIE,"%s",MSG_PROBLEME_CALCUL);
     return(-1);
     }
   return(1);
   }
 double evaluation_fd_beta(int n,double* x,int* test){
   *test = parametre_beta(n-1,&(x[1]));
   if((*test)== -1) return(0.0);
   return(gsl_ran_beta_pdf(x[0],x[1],x[2]));
   } 
 double evaluation_fp_beta(int n,double* x,int* test){
   *test = parametre_beta(n-1,&(x[1]));
   if((*test)== -1) return(0.0);
   return(gsl_cdf_beta_P(x[0],x[1],x[2]));
   }        
 double evaluation_fr_beta(int n,double* x,int* test){
   *test = parametre_beta(n,x);
   if((*test)== -1) return(0.0);
   return(gsl_ran_beta(RNG,x[0],x[1]));
   }        
    
 //********************************************************************************************************
 //construction du tableau de spcification
 //********************************************************************************************************
 //gnrateur de nombre alatoire
//------------------------------------------------------------------------
//routine d'allocation du gnrateur de nombre alatoire
void allocation_rng() {
  //gnrateur employ
  const gsl_rng_type *T    = gsl_rng_taus;
  RNG  = gsl_rng_alloc(T); 
  //graine fixe en fonction de l'horloge
  time_t temps;
  time(&temps);
  long unsigned int s = (long unsigned int) temps;
  //fixe la graine du gnrateur
  gsl_rng_set(RNG,s);
  }

//routine de destruction du gnrateur de nombre alatoire	
void detruit_rng() {
	gsl_rng_free(RNG);
	}
	
 int  ajout_specification_fonction(int pos,char* nom,char type,char symb,int nmin,int nmax,func_pt func) {
   if(pos<0 || pos>NB_FONC) {printf("ERREUR ajout_specification_fonction - position incorrecte !\n"); exit(-1);}
   strcpy(SPEC_FONC_NOM[pos],nom);
   SPEC_FONC_TYPE[pos] = type; 
   SPEC_FONC_SYMB[pos] = symb; 
   SPEC_FONC_NMIN[pos] = nmin; 
   SPEC_FONC_NMAX[pos] = nmax;
   SPEC_FONC_F[pos]    = func;
   return(1);
   }
 
 void charge_specification_fonction() {
 
 int pos = 0;
 
 //allocation du gnrateur de nombre alatoire
 allocation_rng();
 
 //oprateurs****************************************
 //**************************************************  
 ajout_specification_fonction(pos,(char*)"+",'o','+',1,2,&evaluation_addition); pos++;
 ajout_specification_fonction(pos,(char*)"-",'o','-',1,2,&evaluation_soustraction); pos++;
 ajout_specification_fonction(pos,(char*)"*",'o','*',2,2,&evaluation_multiplication); pos++;
 ajout_specification_fonction(pos,(char*)"/",'o','/',2,2,&evaluation_division); pos++;
 ajout_specification_fonction(pos,(char*)"^",'o','^',2,2,&evaluation_puissance); pos++;
 
   
 //fonctions*****************************************
 //**************************************************
 
 //exponentielle
 ajout_specification_fonction(pos,(char*)"exp",'f','e',1,1,&evaluation_exp); pos++;  
 //log
 ajout_specification_fonction(pos,(char*)"log",'f','l',1,1,&evaluation_log); pos++; 
 //valeur absolue
 ajout_specification_fonction(pos,(char*)"abs",'f','a',1,1,&evaluation_abs); pos++;  
 //sinus
 ajout_specification_fonction(pos,(char*)"sin",'f','i',1,1,&evaluation_sin); pos++; 
 //cosinus
 ajout_specification_fonction(pos,(char*)"cos",'f','o',1,1,&evaluation_cos); pos++;  
 //racine
 ajout_specification_fonction(pos,(char*)"sqrt",'f','r',1,1,&evaluation_racine); pos++; 
 //factoriel
 ajout_specification_fonction(pos,(char*)"fact",'f','!',1,1,&evaluation_factoriel); pos++;  
 //somme
 ajout_specification_fonction(pos,(char*)"somme",'f','s',0,-1,&evaluation_addition); pos++; 
 //produit
 ajout_specification_fonction(pos,(char*)"produit",'f','p',0,-1,&evaluation_multiplication); pos++;
 //produit
 ajout_specification_fonction(pos,(char*)"max",'f','<',2,-1,&evaluation_maximum); pos++;
 ajout_specification_fonction(pos,(char*)"min",'f','<',2,-1,&evaluation_minimum); pos++;
 
 char symb = 123;
 //distributions probabilistes discretes   
 //uniforme
 ajout_specification_fonction(pos,(char*)"duniformed",'f',symb,1,3,&evaluation_fd_uniforme_discrete); pos++; symb++; 
 ajout_specification_fonction(pos,(char*)"puniformed",'f',symb,1,3,&evaluation_fp_uniforme_discrete); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"runiformed",'f',symb,0,2,&evaluation_fr_uniforme_discrete); pos++;  symb++;
 //poisson
 ajout_specification_fonction(pos,(char*)"dpoisson",'f',symb,2,2,&evaluation_fd_poisson); pos++; symb++; 
 ajout_specification_fonction(pos,(char*)"ppoisson",'f',symb,2,2,&evaluation_fp_poisson); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rpoisson",'f',symb,1,1,&evaluation_fr_poisson); pos++;  symb++;
 //bernouilli
 ajout_specification_fonction(pos,(char*)"dbernoulli",'f',symb,2,2,&evaluation_fd_bernoulli); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"pbernoulli",'f',symb,2,2,&evaluation_fp_bernoulli); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rbernoulli",'f',symb,1,1,&evaluation_fr_bernoulli); pos++; symb++;
 //bernouilli multiple
 ajout_specification_fonction(pos,(char*)"ddiscrete",'f',symb,2,-1,&evaluation_fd_ddiscrete); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"pdiscrete",'f',symb,2,-1,&evaluation_fp_ddiscrete); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rdiscrete",'f',symb,1,-1,&evaluation_fr_ddiscrete); pos++; symb++;
 //binomiale
 ajout_specification_fonction(pos,(char*)"dbinomiale",'f',symb,3,3,&evaluation_fd_binomiale); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"pbinomiale",'f',symb,3,3,&evaluation_fp_binomiale); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rbinomiale",'f',symb,2,2,&evaluation_fr_binomiale); pos++; symb++; 
 //hypergeometrique
 ajout_specification_fonction(pos,(char*)"dhypergeo",'f',symb,4,4,&evaluation_fd_hypergeometrique); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"phypergeo",'f',symb,4,4,&evaluation_fp_hypergeometrique); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rhypergeo",'f',symb,3,3,&evaluation_fr_hypergeometrique); pos++; symb++;
 //binomiale negative
 ajout_specification_fonction(pos,(char*)"dbinomialeneg",'f',symb,3,3,&evaluation_fd_binomiale_negative); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"pbinomialeneg",'f',symb,3,3,&evaluation_fp_binomiale_negative); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rbinomialeneg",'f',symb,2,2,&evaluation_fr_binomiale_negative); pos++; symb++;
 //geomtrique
 ajout_specification_fonction(pos,(char*)"dgeometrique",'f',symb,2,2,&evaluation_fd_geometrique); pos++; symb++; 
 ajout_specification_fonction(pos,(char*)"pgeometrique",'f',symb,2,2,&evaluation_fp_geometrique); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rgeometrique",'f',symb,1,1,&evaluation_fr_geometrique); pos++; symb++;   
 //distributions probabilistes continues
 //uniforme
 ajout_specification_fonction(pos,(char*)"duniformec",'f',symb,1,3,&evaluation_fd_uniforme_continue); pos++; symb++; 
 ajout_specification_fonction(pos,(char*)"puniformec",'f',symb,1,3,&evaluation_fp_uniforme_continue); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"runiformec",'f',symb,0,2,&evaluation_fr_uniforme_continue); pos++; symb++; 
 //normale
 ajout_specification_fonction(pos,(char*)"dnormale",'f',symb,3,3,&evaluation_fd_normale); pos++; symb++; 
 ajout_specification_fonction(pos,(char*)"pnormale",'f',symb,3,3,&evaluation_fp_normale); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rnormale",'f',symb,2,2,&evaluation_fr_normale); pos++; symb++;     
 //log normale
 ajout_specification_fonction(pos,(char*)"dlognormale",'f',symb,3,3,&evaluation_fd_lognormale); pos++; symb++; 
 ajout_specification_fonction(pos,(char*)"plognormale",'f',symb,3,3,&evaluation_fp_lognormale); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rlognormale",'f',symb,2,2,&evaluation_fr_lognormale); pos++; symb++;       
 //normale
 ajout_specification_fonction(pos,(char*)"dexponentielle",'f',symb,2,2,&evaluation_fd_exponentielle); pos++; symb++; 
 ajout_specification_fonction(pos,(char*)"pexponentielle",'f',symb,2,2,&evaluation_fp_exponentielle); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rexponentielle",'f',symb,1,1,&evaluation_fr_exponentielle); pos++; symb++;    
 //gamma
 ajout_specification_fonction(pos,(char*)"dgamma",'f',symb,3,3,&evaluation_fd_gamma); pos++; symb++; 
 ajout_specification_fonction(pos,(char*)"pgamma",'f',symb,3,3,&evaluation_fp_gamma); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rgamma",'f',symb,2,2,&evaluation_fr_gamma); pos++; symb++;
 //beta 
 ajout_specification_fonction(pos,(char*)"dbeta",'f',symb,3,3,&evaluation_fd_beta); pos++; symb++; 
 ajout_specification_fonction(pos,(char*)"pbeta",'f',symb,3,3,&evaluation_fp_beta); pos++; symb++;
 ajout_specification_fonction(pos,(char*)"rbeta",'f',symb,2,2,&evaluation_fr_beta); pos++; symb++;           
 
 for(;pos<NB_FONC;pos++) SPEC_FONC_TYPE[pos]='?';
 }
  
 //****************************************************
 //routines de base
 //****************************************************
 char symbole_fonction(char* nom){
   unsigned int i =0;
   while(i<NB_FONC && strcmp(SPEC_FONC_NOM[i],nom)!=0) i++;
   if(strcmp(SPEC_FONC_NOM[i],nom)==0) return(SPEC_FONC_SYMB[i]);
   return('?');
   };
   
 char* nom_fonction(char c) {
   unsigned int i =0;
   while(i<NB_FONC && SPEC_FONC_SYMB[i]!=(unsigned int)c) i++;
   if(SPEC_FONC_SYMB[i]==(unsigned int)c) return(SPEC_FONC_NOM[i]);
   return(NULL);
   };
   
 char type(char* nom){
   unsigned int i =0;
   while(i<NB_FONC && strcmp(SPEC_FONC_NOM[i],nom)!=0) i++;
   if(strcmp(SPEC_FONC_NOM[i],nom)==0) return(SPEC_FONC_TYPE[i]);
   return('?');
   };
 
 char type(char  c) {
   unsigned int i =0;
   while(i<NB_FONC && SPEC_FONC_SYMB[i]!=(unsigned int)c) i++;
   if(SPEC_FONC_SYMB[i]==(unsigned int)c) return(SPEC_FONC_TYPE[i]);
   return('?');
   };
 
 char type(int   i) {
   if(i<NB_FONC) return(SPEC_FONC_TYPE[i]);
   return('?');
   }
 
 int nb_arg_min(char* nom){
   unsigned int i =0;
   while(i<NB_FONC && strcmp(SPEC_FONC_NOM[i],nom)!=0) i++;
   if(strcmp(SPEC_FONC_NOM[i],nom)==0) return(SPEC_FONC_NMIN[i]);
   return(-1);
   };
 int nb_arg_min(char  c){
   unsigned int i =0;
   while(i<NB_FONC && SPEC_FONC_SYMB[i]!=(unsigned int)c) i++;
   if(SPEC_FONC_SYMB[i]==(unsigned int)c) return(SPEC_FONC_NMIN[i]);
   return(-1);
   };
 int nb_arg_min(int   i){
   if(i<NB_FONC) return(SPEC_FONC_NMIN[i]);
   return(-1);
   };
 
 int nb_arg_max(char* nom){
   unsigned int i =0;
   while(i<NB_FONC && strcmp(SPEC_FONC_NOM[i],nom)!=0) i++;
   if(strcmp(SPEC_FONC_NOM[i],nom)==0) return(SPEC_FONC_NMAX[i]);
   return(-1);
   };
   
 int nb_arg_max(char  c){
   unsigned int i =0;
   while(i<NB_FONC && SPEC_FONC_SYMB[i]!=(unsigned int)c) i++;
   if(SPEC_FONC_SYMB[i]==(unsigned int)c) return(SPEC_FONC_NMAX[i]);
   return(-1);
   };
   
 int nb_arg_max(int i){
   if(i<NB_FONC) return(SPEC_FONC_NMAX[i]);
   return(-1);
   };
 
 int nb_arg_correct(char* nom,int n){
   unsigned int i =0;
   while(i<NB_FONC && strcmp(SPEC_FONC_NOM[i],nom)!=0) i++;
   if(strcmp(SPEC_FONC_NOM[i],nom)==0 && i<NB_FONC) {
     if((unsigned int) n >= SPEC_FONC_NMIN[i] && (unsigned int) n <= SPEC_FONC_NMAX[i]) return(1);
     else return(0);
     }
   return(-1);
   };
 int nb_arg_correct(char  c,int n){
   unsigned int i =0;
   while(i<NB_FONC && SPEC_FONC_SYMB[i]!= (unsigned int)c) i++;
   if(SPEC_FONC_SYMB[i]==(unsigned int)c && i<NB_FONC) {
     if((unsigned int)n>=SPEC_FONC_NMIN[i] &&(unsigned int)n <=SPEC_FONC_NMAX[i]) return(1);
     else return(0);
     }
   return(-1);
   };
 int nb_arg_correct(int i,int n){
   if(i<NB_FONC) {
     if((unsigned int) n >= SPEC_FONC_NMIN[i] && (unsigned int) n <= SPEC_FONC_NMAX[i]) return(1);
     else return(0);
     }
   return(-1);
   };
 
 func_pt get_fonction_evaluation(char c) {
   unsigned int i =0;
   while(i<NB_FONC && SPEC_FONC_SYMB[i]!=(unsigned int)c) i++;
   if(SPEC_FONC_SYMB[i]==(unsigned int)c && i<NB_FONC) {
     return(SPEC_FONC_F[i]);      
     }
   return(NULL);
 }
   
 int set_fonction_evaluation(double (*f)(int,double*,int*),char* nom) {
   unsigned int i =0;
   while(i<NB_FONC && strcmp(SPEC_FONC_NOM[i],nom)!=0) i++;
   if(strcmp(SPEC_FONC_NOM[i],nom)==0 && i<NB_FONC) {
     f = SPEC_FONC_F[i]; 
     return(1);
     }
   return(-1);
 }
 int set_fonction_evaluation(double (*f)(int,double*,int*),char c) {
   unsigned int i =0;
   while(i<NB_FONC && SPEC_FONC_SYMB[i]!=(unsigned int)c) i++;
   if(SPEC_FONC_SYMB[i]==(unsigned int)c && i<NB_FONC) {
     f = SPEC_FONC_F[i]; 
     return(1);
     }
   return(-1);
 }
 int fonction_evaluation(double (*f)(int,double*,int*),int i) {
   if(i<NB_FONC) {f = SPEC_FONC_F[i]; return(1);}
   return(-1);
   }
   
void ecrire_index_fonctions(char* nom) {
FILE* fichier = fopen(nom,"w");
fprintf(fichier,"<table width=\"100%\" border=\"0\" align=\"left\"><tbody valign=\"center\">\n");
fprintf(fichier,"<tr bgcolor=\"Gray\"><td width=\"30%\">Nom</td><td>Nombre d'arguments min</td><td>Nombre d'arguments max</td>\n");
int cmpt=0;
for(int i=0;i<NB_FONC;i++) {
  if(SPEC_FONC_TYPE[i]!='?') {
   if(cmpt<5) fprintf(fichier,"<tr>"); else fprintf(fichier,"<tr bgcolor=\"lightGray\">");
   cmpt++; if(cmpt==10) cmpt=0;
   fprintf(fichier,"<td>%s</td>",SPEC_FONC_NOM[i]);
   if(SPEC_FONC_NMIN[i]>0) fprintf(fichier,"<td>%d</td>",SPEC_FONC_NMIN[i]);
   else                    fprintf(fichier,"<td> - </td>");
   if(SPEC_FONC_NMAX[i]>0) fprintf(fichier,"<td>%d</td>",SPEC_FONC_NMAX[i]);
   else                    fprintf(fichier,"<td> - </td>");
   fprintf(fichier,"</tr>\n");
   }
  }
fprintf(fichier,"</tbody>\n</table>\n");
fclose(fichier);
}
//
