/***************************************************************************
                          trajectoire.cpp  -  description
                             -------------------
    begin                : dim fv 29 2004
    copyright            : (C) 2004 by Ferdy Jean-Baptiste
    email                : ferdy@univ-montp2.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <iostream.h>
#include <stdlib.h>

#include "trajectoire.h"

//constructeur et destructeur
trajectoire::trajectoire(int l_var_nb,int l_pt_nb,double** l_v) {
  var_nb=l_var_nb;
  pt_nb=l_pt_nb;
  v=l_v;
  _style = 0;
  }

trajectoire::~trajectoire() {
  for(int i=0;i<pt_nb;i++) free(v[i]);
  free(v); v=NULL;
  var_nb=0; pt_nb=0;
  _style = 0;
  }
//routines courantes
double    trajectoire::get_tmax() {
  return(v[pt_nb-1][0]);
  };
void      trajectoire::get_ini(double* x) {
  for(int i=0;i<var_nb;i++) x[i]=v[0][i+1];
  };
  
double   trajectoire::get_xmax() {
  double x=0;
  for(int i=0;i<pt_nb;i++) {
    for(int j=0;j<var_nb;j++) {if(v[i][j+1]>x) x=v[i][j+1];}
    }
  return(x);
  }

double   trajectoire::get_xmax(int j) {
  if(j>var_nb) return(0.0);
  
  double x=0.0;
  for(int i=0;i<pt_nb;i++) if(v[i][j+1]>x) x=v[i][j+1];    
  return(x);
  }

double   trajectoire::get_xmin() {
  double x=v[0][1];
  for(int i=0;i<pt_nb;i++) {
    for(int j=0;j<var_nb;j++) {if(v[i][j+1]<x) x=v[i][j+1];}
    }
  return(x);
  }
      
double   trajectoire::get_xmin(int j) {
  if(j>var_nb) return(0.0);
  
  double x=v[0][j+1];
  for(int i=1;i<pt_nb;i++) if(v[i][j+1]<x) x=v[i][j+1];    
  return(x);
  }
  

//criture dans la sortie standard
void trajectoire::ecrire() {
  for(int i=0;i<pt_nb;i++) {
    for(int j=0;j<var_nb;j++) printf("%g\t",v[i][j]);
    printf("%g\n",v[i][var_nb]);
    }
  }

void trajectoire::ecrire_sans_t() {
  for(int i=0;i<pt_nb;i++) {
    for(int j=1;j<var_nb;j++) printf("%g\t",v[i][j]);
    printf("%g\n",v[i][var_nb]);
    }
  }
  
//criture dans un fichier
void trajectoire::ecrire(FILE* fichier) {
  for(int i=0;i<pt_nb;i++) {
    for(int j=0;j<var_nb;j++) fprintf(fichier,"%g\t",v[i][j]);
    fprintf(fichier,"%g\n",v[i][var_nb]);    
    }
  }

void trajectoire::ecrire_sans_t(FILE* fichier) {
  for(int i=0;i<pt_nb;i++) {
    for(int j=1;j<var_nb;j++) fprintf(fichier,"%g\t",v[i][j]);
    fprintf(fichier,"%g\n",v[i][var_nb]);    
    }
  }
      
//criture du point i dans un fichier  
void trajectoire::ecrire(FILE* fichier,int i,char c) {  
    if(i>=pt_nb) {
      for(int j=0;j<var_nb;j++) fprintf(fichier,".\t");
      fprintf(fichier,".%c",c);
      }
    else {
      for(int j=0;j<var_nb;j++) fprintf(fichier,"%g\t",v[i][j]);   
      fprintf(fichier,"%g%c",v[i][var_nb],c);
      }
  }    

//criture du point i dans un fichier  
void trajectoire::ecrire_sans_t(FILE* fichier,int i,char c) {
    if(i>=pt_nb) {
      for(int j=1;j<var_nb;j++) fprintf(fichier,".\t");
      fprintf(fichier,".%c",c);
      }
    else {
      for(int j=1;j<var_nb;j++) fprintf(fichier,"%g\t",v[i][j]);
      fprintf(fichier,"%g%c",v[i][var_nb],c);
      }
  }

//calcule la distribution de la variable i entre t1 et t2
//et renvoit un histogame GSL
gsl_histogram*  trajectoire::histograme(double t1,double t2,int var_pos,int nb_classe)
{
  gsl_histogram * h = gsl_histogram_alloc (nb_classe);
  gsl_histogram_set_ranges_uniform (h,get_xmin(var_pos),get_xmax(var_pos));
  for(int i=0;i<pt_nb;i++) {
    double z = v[var_pos][i];
    if(z>=t1 && z<=t2) gsl_histogram_increment (h,z);
    }
  return(h);
}